"""Node.js 安装器模块

该模块实现 Node.js 的安装、升级和验证逻辑。
Node.js 通过 NVM 进行安装和管理。
"""

from pathlib import Path
from typing import Optional

from ..config import ToolConfig
from ..installer_base import InstallReport, InstallResult, ToolInstaller
from ..platform_detector import PlatformInfo


class NodeInstaller(ToolInstaller):
    """Node.js 安装器
    
    负责安装、升级和验证 Node.js。
    Node.js 通过 NVM 安装 LTS 版本，并设置为默认版本。
    
    Attributes:
        platform_info: 平台信息
        config: 工具配置
        node_version: Node.js 版本（默认 'lts/*' 表示最新 LTS 版本）
    """
    
    def __init__(self, platform_info: PlatformInfo, config: ToolConfig):
        """初始化 Node.js 安装器
        
        Args:
            platform_info: 平台信息
            config: 工具配置
        """
        super().__init__(platform_info, config)
        
        # 从配置中获取版本，如果未指定则使用 LTS
        self.node_version = config.version or "lts/*"
    
    def _check_nvm_installed(self) -> bool:
        """检查 NVM 是否已安装
        
        Returns:
            bool: 如果 NVM 已安装返回 True，否则返回 False
        """
        nvm_dir = Path.home() / ".nvm"
        nvm_sh = nvm_dir / "nvm.sh"
        
        if not nvm_sh.exists():
            return False
        
        # 尝试执行 nvm --version 命令
        returncode, stdout, stderr = self.run_command(
            f"bash -c 'source {nvm_sh} && nvm --version'",
            shell=True,
            timeout=10,
            max_retries=1
        )
        
        return returncode == 0
    
    def verify(self) -> bool:
        """验证 Node.js 是否已正确安装
        
        检查 node 命令是否可用，并尝试执行 node --version 命令。
        
        Returns:
            bool: 如果验证成功返回 True，否则返回 False
        """
        nvm_sh = Path.home() / ".nvm" / "nvm.sh"
        
        if not nvm_sh.exists():
            return False
        
        # 尝试执行 node --version 命令
        returncode, stdout, stderr = self.run_command(
            f"bash -c 'source {nvm_sh} && node --version'",
            shell=True,
            timeout=10,
            max_retries=1
        )
        
        return returncode == 0
    
    def _get_installed_version(self) -> Optional[str]:
        """获取已安装的 Node.js 版本
        
        Returns:
            Optional[str]: 版本号，如果无法获取则返回 None
        """
        nvm_sh = Path.home() / ".nvm" / "nvm.sh"
        
        if not nvm_sh.exists():
            return None
        
        returncode, stdout, stderr = self.run_command(
            f"bash -c 'source {nvm_sh} && node --version'",
            shell=True,
            timeout=10,
            max_retries=1
        )
        
        if returncode == 0:
            return stdout.strip()
        
        return None
    
    def _install_node_via_nvm(self) -> tuple[bool, str]:
        """通过 NVM 安装 Node.js
        
        Returns:
            tuple[bool, str]: (是否成功, 错误信息)
        """
        nvm_sh = Path.home() / ".nvm" / "nvm.sh"
        
        # 安装 Node.js
        install_cmd = f"bash -c 'source {nvm_sh} && nvm install {self.node_version}'"
        returncode, stdout, stderr = self.run_command(
            install_cmd,
            shell=True,
            timeout=600,  # Node.js 安装可能需要较长时间
            max_retries=2
        )
        
        if returncode != 0:
            return False, stderr or "安装命令返回非零退出码"
        
        return True, ""
    
    def _set_default_version(self) -> tuple[bool, str]:
        """设置 Node.js 默认版本
        
        Returns:
            tuple[bool, str]: (是否成功, 错误信息)
        """
        nvm_sh = Path.home() / ".nvm" / "nvm.sh"
        
        # 设置默认版本
        # 如果安装的是 lts/*，则将默认版本设置为 lts/*
        # 否则设置为指定的版本
        if self.node_version == "lts/*":
            default_version = "lts/*"
        else:
            default_version = self.node_version
        
        alias_cmd = f"bash -c 'source {nvm_sh} && nvm alias default {default_version}'"
        returncode, stdout, stderr = self.run_command(
            alias_cmd,
            shell=True,
            timeout=30,
            max_retries=1
        )
        
        if returncode != 0:
            return False, stderr or "设置默认版本命令返回非零退出码"
        
        return True, ""
    
    def install(self) -> InstallReport:
        """安装 Node.js
        
        执行以下步骤：
        1. 检查 NVM 是否已安装（Node.js 依赖 NVM）
        2. 检查 Node.js 是否已安装
        3. 通过 NVM 安装 Node.js LTS 版本
        4. 设置为默认版本
        5. 验证安装
        
        Returns:
            InstallReport: 安装报告
        """
        # 检查 NVM 是否已安装
        if not self._check_nvm_installed():
            return InstallReport(
                tool_name="node",
                result=InstallResult.FAILED,
                message="NVM 未安装，无法安装 Node.js",
                error="请先安装 NVM"
            )
        
        # 检查是否已安装
        if self.verify():
            version = self._get_installed_version()
            return InstallReport(
                tool_name="node",
                result=InstallResult.SKIPPED,
                message="Node.js 已安装，跳过安装",
                version=version
            )
        
        try:
            # 通过 NVM 安装 Node.js
            install_success, install_error = self._install_node_via_nvm()
            
            if not install_success:
                return InstallReport(
                    tool_name="node",
                    result=InstallResult.FAILED,
                    message="通过 NVM 安装 Node.js 失败",
                    error=install_error
                )
            
            # 设置默认版本
            default_success, default_error = self._set_default_version()
            
            if not default_success:
                return InstallReport(
                    tool_name="node",
                    result=InstallResult.FAILED,
                    message="设置 Node.js 默认版本失败",
                    error=default_error
                )
            
            # 验证安装
            if not self.verify():
                return InstallReport(
                    tool_name="node",
                    result=InstallResult.FAILED,
                    message="Node.js 安装验证失败",
                    error="安装命令执行成功，但无法验证 Node.js 是否正确安装"
                )
            
            # 获取安装的版本
            version = self._get_installed_version()
            
            return InstallReport(
                tool_name="node",
                result=InstallResult.SUCCESS,
                message="Node.js 安装成功",
                version=version
            )
            
        except Exception as e:
            return InstallReport(
                tool_name="node",
                result=InstallResult.FAILED,
                message="Node.js 安装过程中发生异常",
                error=str(e)
            )
    
    def upgrade(self) -> InstallReport:
        """升级 Node.js
        
        执行以下步骤：
        1. 检查 NVM 是否已安装
        2. 检查 Node.js 是否已安装
        3. 通过 NVM 安装最新的 LTS 版本
        4. 设置为默认版本
        5. 验证升级
        
        Returns:
            InstallReport: 升级报告
        """
        # 检查 NVM 是否已安装
        if not self._check_nvm_installed():
            return InstallReport(
                tool_name="node",
                result=InstallResult.FAILED,
                message="NVM 未安装，无法升级 Node.js",
                error="请先安装 NVM"
            )
        
        # 检查是否已安装
        if not self.verify():
            return InstallReport(
                tool_name="node",
                result=InstallResult.FAILED,
                message="Node.js 未安装，无法升级",
                error="请先安装 Node.js"
            )
        
        try:
            # 记录升级前的版本
            old_version = self._get_installed_version()
            
            # 通过 NVM 安装最新的 LTS 版本（会自动下载新版本）
            install_success, install_error = self._install_node_via_nvm()
            
            if not install_success:
                return InstallReport(
                    tool_name="node",
                    result=InstallResult.FAILED,
                    message="通过 NVM 升级 Node.js 失败",
                    error=install_error
                )
            
            # 设置默认版本
            default_success, default_error = self._set_default_version()
            
            if not default_success:
                return InstallReport(
                    tool_name="node",
                    result=InstallResult.FAILED,
                    message="设置 Node.js 默认版本失败",
                    error=default_error
                )
            
            # 验证升级
            if not self.verify():
                return InstallReport(
                    tool_name="node",
                    result=InstallResult.FAILED,
                    message="Node.js 升级验证失败",
                    error="升级命令执行成功，但无法验证 Node.js 是否正确安装"
                )
            
            # 获取升级后的版本
            new_version = self._get_installed_version()
            
            return InstallReport(
                tool_name="node",
                result=InstallResult.SUCCESS,
                message=f"Node.js 升级成功 ({old_version} -> {new_version})",
                version=new_version
            )
            
        except Exception as e:
            return InstallReport(
                tool_name="node",
                result=InstallResult.FAILED,
                message="Node.js 升级过程中发生异常",
                error=str(e)
            )



"""单元测试：Codex CLI 安装器

该模块测试 Codex CLI 安装器的各种场景，包括安装方式选择逻辑。
"""

import pytest
from unittest.mock import Mock, patch, MagicMock
from pathlib import Path

from src.mono_kickstart.config import ToolConfig
from src.mono_kickstart.installer_base import InstallResult
from src.mono_kickstart.installers.codex_installer import CodexInstaller
from src.mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell


@pytest.fixture
def platform_info():
    """创建测试用的平台信息"""
    return PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file="/home/test/.bashrc"
    )


@pytest.fixture
def tool_config():
    """创建测试用的工具配置"""
    return ToolConfig(enabled=True)


@pytest.fixture
def installer(platform_info, tool_config):
    """创建 Codex CLI 安装器实例"""
    with patch('shutil.which', return_value=None):
        return CodexInstaller(platform_info, tool_config)


class TestCodexInstallerDetermineInstallMethod:
    """测试 Codex CLI 安装器的安装方式选择逻辑"""
    
    def test_determine_method_when_bun_installed(self, platform_info):
        """测试 Bun 已安装时选择 Bun"""
        config = ToolConfig(enabled=True)
        with patch('shutil.which', return_value="/usr/local/bin/bun"):
            installer = CodexInstaller(platform_info, config)
            assert installer.install_method == 'bun'
    
    def test_determine_method_when_bun_not_installed(self, platform_info):
        """测试 Bun 未安装时选择 npm"""
        config = ToolConfig(enabled=True)
        with patch('shutil.which', return_value=None):
            installer = CodexInstaller(platform_info, config)
            assert installer.install_method == 'npm'
    
    def test_determine_method_with_config_override_bun(self, platform_info):
        """测试配置指定使用 Bun"""
        config = ToolConfig(enabled=True, install_via='bun')
        with patch('shutil.which', return_value=None):
            installer = CodexInstaller(platform_info, config)
            assert installer.install_method == 'bun'
    
    def test_determine_method_with_config_override_npm(self, platform_info):
        """测试配置指定使用 npm"""
        config = ToolConfig(enabled=True, install_via='npm')
        with patch('shutil.which', return_value="/usr/local/bin/bun"):
            installer = CodexInstaller(platform_info, config)
            assert installer.install_method == 'npm'
    
    def test_determine_method_with_config_case_insensitive(self, platform_info):
        """测试配置大小写不敏感"""
        config = ToolConfig(enabled=True, install_via='BUN')
        with patch('shutil.which', return_value=None):
            installer = CodexInstaller(platform_info, config)
            assert installer.install_method == 'bun'


class TestCodexInstallerVerify:
    """测试 Codex CLI 安装器的验证功能"""
    
    def test_verify_when_codex_installed(self, installer):
        """测试 Codex CLI 已安装时的验证"""
        with patch('shutil.which', return_value="/usr/local/bin/codex"), \
             patch.object(installer, 'run_command', return_value=(0, "1.0.0", "")):
            assert installer.verify() is True
    
    def test_verify_when_codex_not_in_path(self, installer):
        """测试 Codex CLI 不在 PATH 中时的验证"""
        with patch('shutil.which', return_value=None):
            assert installer.verify() is False
    
    def test_verify_when_codex_command_fails(self, installer):
        """测试 Codex CLI 命令执行失败时的验证"""
        with patch('shutil.which', return_value="/usr/local/bin/codex"), \
             patch.object(installer, 'run_command', return_value=(1, "", "error")):
            assert installer.verify() is False


class TestCodexInstallerGetVersion:
    """测试 Codex CLI 安装器的版本获取功能"""
    
    def test_get_version_success(self, installer):
        """测试成功获取版本"""
        with patch('shutil.which', return_value="/usr/local/bin/codex"), \
             patch.object(installer, 'run_command', return_value=(0, "2.1.0", "")):
            version = installer._get_installed_version()
            assert version == "2.1.0"
    
    def test_get_version_when_not_installed(self, installer):
        """测试 Codex CLI 未安装时获取版本"""
        with patch('shutil.which', return_value=None):
            version = installer._get_installed_version()
            assert version is None
    
    def test_get_version_when_command_fails(self, installer):
        """测试命令失败时获取版本"""
        with patch('shutil.which', return_value="/usr/local/bin/codex"), \
             patch.object(installer, 'run_command', return_value=(1, "", "error")):
            version = installer._get_installed_version()
            assert version is None


class TestCodexInstallerInstall:
    """测试 Codex CLI 安装器的安装功能"""
    
    def test_install_when_already_installed(self, installer):
        """测试 Codex CLI 已安装时跳过安装"""
        with patch.object(installer, 'verify', return_value=True), \
             patch.object(installer, '_get_installed_version', return_value="2.0.0"):
            report = installer.install()
            
            assert report.result == InstallResult.SKIPPED
            assert report.tool_name == "codex"
            assert report.version == "2.0.0"
            assert "已安装" in report.message
    
    def test_install_with_bun_success(self, platform_info):
        """测试使用 Bun 成功安装 Codex CLI"""
        config = ToolConfig(enabled=True, install_via='bun')
        
        with patch('shutil.which') as mock_which, \
             patch.object(CodexInstaller, 'verify', side_effect=[False, True]), \
             patch.object(CodexInstaller, 'run_command', return_value=(0, "installed", "")), \
             patch.object(CodexInstaller, '_get_installed_version', return_value="2.1.0"):
            
            def which_side_effect(cmd):
                if cmd == "bun":
                    return "/usr/local/bin/bun"
                elif cmd == "codex":
                    return None
                return None
            
            mock_which.side_effect = which_side_effect
            
            installer = CodexInstaller(platform_info, config)
            report = installer.install()
            
            assert report.result == InstallResult.SUCCESS
            assert report.tool_name == "codex"
            assert report.version == "2.1.0"
            assert "bun" in report.message.lower()
    
    def test_install_with_npm_success(self, platform_info):
        """测试使用 npm 成功安装 Codex CLI"""
        config = ToolConfig(enabled=True, install_via='npm')
        
        with patch('shutil.which') as mock_which, \
             patch.object(CodexInstaller, 'verify', side_effect=[False, True]), \
             patch.object(CodexInstaller, 'run_command', return_value=(0, "installed", "")), \
             patch.object(CodexInstaller, '_get_installed_version', return_value="2.1.0"):
            
            def which_side_effect(cmd):
                if cmd == "npm":
                    return "/usr/local/bin/npm"
                elif cmd == "codex":
                    return None
                return None
            
            mock_which.side_effect = which_side_effect
            
            installer = CodexInstaller(platform_info, config)
            report = installer.install()
            
            assert report.result == InstallResult.SUCCESS
            assert report.tool_name == "codex"
            assert report.version == "2.1.0"
            assert "npm" in report.message.lower()
    
    def test_install_with_bun_but_bun_not_available(self, platform_info):
        """测试配置使用 Bun 但 Bun 未安装"""
        config = ToolConfig(enabled=True, install_via='bun')
        
        with patch('shutil.which', return_value=None), \
             patch.object(CodexInstaller, 'verify', return_value=False):
            
            installer = CodexInstaller(platform_info, config)
            report = installer.install()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "codex"
            assert "Bun 未安装" in report.message
    
    def test_install_with_npm_but_npm_not_available(self, platform_info):
        """测试配置使用 npm 但 npm 未安装"""
        config = ToolConfig(enabled=True, install_via='npm')
        
        with patch('shutil.which', return_value=None), \
             patch.object(CodexInstaller, 'verify', return_value=False):
            
            installer = CodexInstaller(platform_info, config)
            report = installer.install()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "codex"
            assert "npm 未安装" in report.message
    
    def test_install_command_fails(self, installer):
        """测试安装命令执行失败"""
        with patch('shutil.which') as mock_which, \
             patch.object(installer, 'verify', return_value=False), \
             patch.object(installer, 'run_command', return_value=(1, "", "install failed")):
            
            mock_which.return_value = "/usr/local/bin/npm"
            
            report = installer.install()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "codex"
            assert "失败" in report.message
    
    def test_install_verification_fails(self, installer):
        """测试安装后验证失败"""
        with patch('shutil.which', return_value="/usr/local/bin/npm"), \
             patch.object(installer, 'verify', side_effect=[False, False]), \
             patch.object(installer, 'run_command', return_value=(0, "installed", "")):
            
            report = installer.install()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "codex"
            assert "验证失败" in report.message
    
    def test_install_exception(self, installer):
        """测试安装过程中发生异常"""
        with patch('shutil.which', return_value="/usr/local/bin/npm"), \
             patch.object(installer, 'verify', return_value=False), \
             patch.object(installer, 'run_command', side_effect=Exception("test error")):
            report = installer.install()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "codex"
            assert "异常" in report.message
            assert "test error" in report.error


class TestCodexInstallerUpgrade:
    """测试 Codex CLI 安装器的升级功能"""
    
    def test_upgrade_when_not_installed(self, installer):
        """测试 Codex CLI 未安装时无法升级"""
        with patch.object(installer, 'verify', return_value=False):
            report = installer.upgrade()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "codex"
            assert "未安装" in report.message
    
    def test_upgrade_with_bun_success(self, platform_info):
        """测试使用 Bun 成功升级 Codex CLI"""
        config = ToolConfig(enabled=True, install_via='bun')
        
        with patch('shutil.which') as mock_which, \
             patch.object(CodexInstaller, 'verify', return_value=True), \
             patch.object(CodexInstaller, '_get_installed_version', side_effect=["2.0.0", "2.1.0"]), \
             patch.object(CodexInstaller, 'run_command', return_value=(0, "upgraded", "")):
            
            def which_side_effect(cmd):
                if cmd == "bun":
                    return "/usr/local/bin/bun"
                elif cmd == "codex":
                    return "/usr/local/bin/codex"
                return None
            
            mock_which.side_effect = which_side_effect
            
            installer = CodexInstaller(platform_info, config)
            report = installer.upgrade()
            
            assert report.result == InstallResult.SUCCESS
            assert report.tool_name == "codex"
            assert report.version == "2.1.0"
            assert "2.0.0" in report.message
            assert "2.1.0" in report.message
    
    def test_upgrade_with_npm_success(self, platform_info):
        """测试使用 npm 成功升级 Codex CLI"""
        config = ToolConfig(enabled=True, install_via='npm')
        
        with patch('shutil.which') as mock_which, \
             patch.object(CodexInstaller, 'verify', return_value=True), \
             patch.object(CodexInstaller, '_get_installed_version', side_effect=["2.0.0", "2.1.0"]), \
             patch.object(CodexInstaller, 'run_command', return_value=(0, "upgraded", "")):
            
            def which_side_effect(cmd):
                if cmd == "npm":
                    return "/usr/local/bin/npm"
                elif cmd == "codex":
                    return "/usr/local/bin/codex"
                return None
            
            mock_which.side_effect = which_side_effect
            
            installer = CodexInstaller(platform_info, config)
            report = installer.upgrade()
            
            assert report.result == InstallResult.SUCCESS
            assert report.tool_name == "codex"
            assert report.version == "2.1.0"
    
    def test_upgrade_command_fails(self, installer):
        """测试升级命令执行失败"""
        with patch('shutil.which', return_value="/usr/local/bin/npm"), \
             patch.object(installer, 'verify', return_value=True), \
             patch.object(installer, '_get_installed_version', return_value="2.0.0"), \
             patch.object(installer, 'run_command', return_value=(1, "", "upgrade failed")):
            
            report = installer.upgrade()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "codex"
            assert "失败" in report.message
    
    def test_upgrade_verification_fails(self, installer):
        """测试升级后验证失败"""
        with patch('shutil.which', return_value="/usr/local/bin/npm"), \
             patch.object(installer, 'verify', side_effect=[True, False]), \
             patch.object(installer, '_get_installed_version', return_value="2.0.0"), \
             patch.object(installer, 'run_command', return_value=(0, "upgraded", "")):
            
            report = installer.upgrade()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "codex"
            assert "验证失败" in report.message
    
    def test_upgrade_exception(self, installer):
        """测试升级过程中发生异常"""
        with patch.object(installer, 'verify', return_value=True), \
             patch.object(installer, '_get_installed_version', side_effect=Exception("test error")):
            report = installer.upgrade()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "codex"
            assert "异常" in report.message
            assert "test error" in report.error



"""配置管理器的基于属性的测试

本模块使用 Hypothesis 框架测试配置管理器的通用属性。
"""

import pytest
from hypothesis import given, strategies as st, settings, HealthCheck
from pathlib import Path
import yaml
import tempfile
import shutil

from mono_kickstart.config import (
    ToolConfig,
    RegistryConfig,
    ProjectConfig,
    Config,
    ConfigManager,
)


# 定义测试数据生成策略

# 有效的工具名称
VALID_TOOLS = [
    "nvm", "node", "conda", "bun", "uv",
    "claude-code", "codex", "spec-kit", "bmad-method"
]

# 有效的安装方式
VALID_INSTALL_METHODS = ["bun", "npm", "brew", None]


@st.composite
def tool_config_strategy(draw):
    """生成 ToolConfig 对象的策略"""
    return ToolConfig(
        enabled=draw(st.booleans()),
        version=draw(st.one_of(
            st.none(), 
            st.text(
                alphabet=st.characters(min_codepoint=32, max_codepoint=126, blacklist_categories=('Cc', 'Cs')),
                min_size=1, 
                max_size=20
            )
        )),
        install_via=draw(st.sampled_from(VALID_INSTALL_METHODS)),
        extra_options=draw(st.dictionaries(
            st.text(
                alphabet=st.characters(min_codepoint=32, max_codepoint=126, blacklist_categories=('Cc', 'Cs')),
                min_size=1, 
                max_size=10
            ),
            st.text(
                alphabet=st.characters(min_codepoint=32, max_codepoint=126, blacklist_categories=('Cc', 'Cs')),
                min_size=1, 
                max_size=20
            ),
            max_size=3
        ))
    )


@st.composite
def registry_config_strategy(draw):
    """生成 RegistryConfig 对象的策略"""
    return RegistryConfig(
        npm=draw(st.text(
            alphabet=st.characters(min_codepoint=32, max_codepoint=126, blacklist_categories=('Cc', 'Cs')),
            min_size=10, 
            max_size=100
        )),
        bun=draw(st.text(
            alphabet=st.characters(min_codepoint=32, max_codepoint=126, blacklist_categories=('Cc', 'Cs')),
            min_size=10, 
            max_size=100
        )),
        pypi=draw(st.text(
            alphabet=st.characters(min_codepoint=32, max_codepoint=126, blacklist_categories=('Cc', 'Cs')),
            min_size=10, 
            max_size=100
        )),
        python_install=draw(st.text(
            alphabet=st.characters(min_codepoint=32, max_codepoint=126, blacklist_categories=('Cc', 'Cs')),
            min_size=10, 
            max_size=100
        ))
    )


@st.composite
def project_config_strategy(draw):
    """生成 ProjectConfig 对象的策略"""
    return ProjectConfig(
        name=draw(st.one_of(
            st.none(), 
            st.text(
                alphabet=st.characters(min_codepoint=32, max_codepoint=126, blacklist_categories=('Cc', 'Cs')),
                min_size=1, 
                max_size=50
            )
        ))
    )


@st.composite
def config_strategy(draw):
    """生成 Config 对象的策略"""
    return Config(
        project=draw(project_config_strategy()),
        tools=draw(st.dictionaries(
            st.sampled_from(VALID_TOOLS),
            tool_config_strategy(),
            max_size=5
        )),
        registry=draw(registry_config_strategy())
    )


class TestConfigPriorityMerge:
    """测试属性 7: 配置优先级合并
    
    **Validates: Requirements 4.1, 4.9**
    """
    
    @given(
        config1=config_strategy(),
        config2=config_strategy(),
        config3=config_strategy(),
        config4=config_strategy()
    )
    def test_merge_priority_order(self, config1, config2, config3, config4):
        """验证配置按优先级正确合并
        
        对于任何多个配置源，配置管理器应该按优先级合并，
        高优先级配置字段覆盖低优先级字段。
        """
        manager = ConfigManager()
        merged = manager.merge_configs(config1, config2, config3, config4)
        
        # 验证 project.name 使用最后一个非 None 的值
        expected_name = None
        for config in [config1, config2, config3, config4]:
            if config.project.name is not None:
                expected_name = config.project.name
        assert merged.project.name == expected_name
        
        # 验证所有工具配置都被包含（后面的覆盖前面的）
        all_tools = set()
        for config in [config1, config2, config3, config4]:
            all_tools.update(config.tools.keys())
        
        for tool_name in all_tools:
            assert tool_name in merged.tools
            
            # 找到最后一个定义该工具的配置
            expected_tool_config = None
            for config in [config1, config2, config3, config4]:
                if tool_name in config.tools:
                    expected_tool_config = config.tools[tool_name]
            
            # 验证合并后的工具配置与预期一致
            assert merged.tools[tool_name] == expected_tool_config
    
    @given(configs=st.lists(config_strategy(), min_size=1, max_size=5))
    def test_merge_is_associative(self, configs):
        """验证配置合并满足结合律
        
        merge(a, b, c) 应该等于 merge(merge(a, b), c)
        """
        manager = ConfigManager()
        
        # 一次性合并所有配置
        result1 = manager.merge_configs(*configs)
        
        # 逐步合并配置
        result2 = configs[0]
        for config in configs[1:]:
            result2 = manager.merge_configs(result2, config)
        
        # 验证结果相同
        assert result1.project.name == result2.project.name
        assert result1.tools.keys() == result2.tools.keys()
        for tool_name in result1.tools:
            assert result1.tools[tool_name] == result2.tools[tool_name]
    
    @given(config=config_strategy())
    def test_merge_with_empty_is_identity(self, config):
        """验证与空配置合并保持不变
        
        merge(config, empty) 应该等于 config
        merge(empty, config) 应该等于 config
        """
        manager = ConfigManager()
        empty = Config()
        
        # 空配置在后
        result1 = manager.merge_configs(config, empty)
        # 空配置在前
        result2 = manager.merge_configs(empty, config)
        
        # 验证 project.name
        assert result1.project.name == config.project.name
        assert result2.project.name == config.project.name
        
        # 验证 tools
        assert result1.tools.keys() == config.tools.keys()
        assert result2.tools.keys() == config.tools.keys()


class TestConfigFileFormatValidation:
    """测试属性 8: 配置文件格式验证
    
    **Validates: Requirements 4.4**
    """
    
    @given(
        invalid_yaml=st.text(min_size=1, max_size=100).filter(
            lambda s: s.strip() and not s.strip().startswith('#')
        )
    )
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
    def test_invalid_yaml_raises_error(self, invalid_yaml):
        """验证无效的 YAML 格式抛出错误
        
        对于任何无效的 YAML 格式配置文件，
        配置管理器应该抛出格式错误或类型错误。
        """
        # 使用临时目录
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            # 创建无效的 YAML 文件
            config_file = tmp_path / "invalid.yaml"
            # 添加一些明显无效的 YAML 语法
            config_file.write_text(f"{invalid_yaml}\n  invalid: yaml: [[[")
            
            manager = ConfigManager()
            
            # 验证抛出 YAML 错误或 AttributeError（当 YAML 解析为非字典类型时）
            with pytest.raises((yaml.YAMLError, AttributeError)):
                manager.load_from_file(config_file)
    
    @given(config=config_strategy())
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
    def test_valid_config_loads_without_error(self, config):
        """验证有效的配置可以正常加载
        
        对于任何有效的配置对象，保存后应该能够无错误加载。
        """
        # 使用临时目录
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            config_file = tmp_path / "valid.yaml"
            manager = ConfigManager()
            
            # 保存配置
            manager.save_to_file(config, config_file)
            
            # 加载配置不应抛出异常
            loaded_config = manager.load_from_file(config_file)
            assert isinstance(loaded_config, Config)


class TestConfigRoundtripConsistency:
    """测试属性 9: 配置往返一致性
    
    **Validates: Requirements 4.5, 11.8**
    """
    
    @given(config=config_strategy())
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
    def test_save_load_roundtrip(self, config):
        """验证配置保存和加载的往返一致性
        
        对于任何有效的配置对象，保存到文件然后重新加载
        应该产生等价的配置对象。
        """
        # 使用临时目录
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            config_file = tmp_path / "config.yaml"
            manager = ConfigManager()
            
            # 保存配置
            manager.save_to_file(config, config_file)
            
            # 加载配置
            loaded_config = manager.load_from_file(config_file)
            
            # 验证 project 配置一致
            assert loaded_config.project.name == config.project.name
            
            # 验证 tools 配置一致
            assert loaded_config.tools.keys() == config.tools.keys()
            for tool_name in config.tools:
                original_tool = config.tools[tool_name]
                loaded_tool = loaded_config.tools[tool_name]
                assert loaded_tool.enabled == original_tool.enabled
                assert loaded_tool.version == original_tool.version
                assert loaded_tool.install_via == original_tool.install_via
                assert loaded_tool.extra_options == original_tool.extra_options
            
            # 验证 registry 配置一致
            assert loaded_config.registry.npm == config.registry.npm
            assert loaded_config.registry.bun == config.registry.bun
            assert loaded_config.registry.pypi == config.registry.pypi
            assert loaded_config.registry.python_install == config.registry.python_install
    
    @given(
        config=config_strategy(),
        iterations=st.integers(min_value=2, max_value=5)
    )
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
    def test_multiple_roundtrips(self, config, iterations):
        """验证多次往返保持一致性
        
        多次保存和加载应该保持配置不变。
        """
        # 使用临时目录
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            config_file = tmp_path / "config.yaml"
            manager = ConfigManager()
            
            current_config = config
            for i in range(iterations):
                # 保存
                manager.save_to_file(current_config, config_file)
                # 加载
                current_config = manager.load_from_file(config_file)
            
            # 验证最终配置与原始配置一致
            assert current_config.project.name == config.project.name
            assert current_config.tools.keys() == config.tools.keys()
            for tool_name in config.tools:
                assert current_config.tools[tool_name].enabled == config.tools[tool_name].enabled
    
    @given(config=config_strategy())
    def test_to_dict_from_dict_roundtrip(self, config):
        """验证字典转换的往返一致性
        
        Config -> dict -> Config 应该保持一致。
        """
        # 转换为字典
        config_dict = config.to_dict()
        
        # 从字典创建配置
        restored_config = Config.from_dict(config_dict)
        
        # 验证一致性
        assert restored_config.project.name == config.project.name
        assert restored_config.tools.keys() == config.tools.keys()
        for tool_name in config.tools:
            assert restored_config.tools[tool_name].enabled == config.tools[tool_name].enabled
            assert restored_config.tools[tool_name].version == config.tools[tool_name].version


class TestConfigValidation:
    """测试配置验证的属性"""
    
    @given(config=config_strategy())
    def test_valid_config_has_no_errors(self, config):
        """验证有效的配置不产生错误
        
        对于任何使用有效工具名称和安装方式的配置，
        验证应该返回空错误列表。
        """
        manager = ConfigManager()
        errors = manager.validate(config)
        
        # 有效配置不应有错误
        assert errors == []
    
    @given(
        config=config_strategy(),
        invalid_tool=st.text(min_size=1, max_size=20).filter(
            lambda s: s not in VALID_TOOLS
        )
    )
    def test_invalid_tool_name_produces_error(self, config, invalid_tool):
        """验证无效的工具名称产生错误
        
        对于任何包含无效工具名称的配置，
        验证应该返回包含该工具名称的错误。
        """
        # 添加无效的工具
        config.tools[invalid_tool] = ToolConfig()
        
        manager = ConfigManager()
        errors = manager.validate(config)
        
        # 应该有错误
        assert len(errors) > 0
        # 错误信息应该包含无效的工具名称
        assert any(invalid_tool in error for error in errors)
    
    @given(
        config=config_strategy(),
        invalid_method=st.text(min_size=1, max_size=20).filter(
            lambda s: s not in ["bun", "npm", "brew"]
        )
    )
    def test_invalid_install_via_produces_error(self, config, invalid_method):
        """验证无效的 install_via 产生错误
        
        对于任何包含无效安装方式的配置，
        验证应该返回相关错误。
        """
        # 添加一个有效工具但使用无效的安装方式
        config.tools["nvm"] = ToolConfig(install_via=invalid_method)
        
        manager = ConfigManager()
        errors = manager.validate(config)
        
        # 应该有错误
        assert len(errors) > 0
        # 错误信息应该提到 install_via
        assert any("install_via" in error for error in errors)



"""Spec Kit 安装器模块

该模块实现 Spec Kit 的安装、升级和验证逻辑。
"""

import re
import shutil
from pathlib import Path
from typing import Optional

from ..config import ToolConfig
from ..installer_base import InstallReport, InstallResult, ToolInstaller
from ..platform_detector import PlatformInfo


class SpecKitInstaller(ToolInstaller):
    """Spec Kit 安装器
    
    负责安装、升级和验证 Spec Kit。
    Spec Kit 通过 uv tool install 从 GitHub 仓库安装。
    
    Attributes:
        platform_info: 平台信息
        config: 工具配置
        github_repo: GitHub 仓库地址
    """
    
    def __init__(self, platform_info: PlatformInfo, config: ToolConfig):
        """初始化 Spec Kit 安装器
        
        Args:
            platform_info: 平台信息
            config: 工具配置
        """
        super().__init__(platform_info, config)
        
        # Spec Kit GitHub 仓库地址
        self.github_repo = "git+https://github.com/github/spec-kit.git"
    
    def verify(self) -> bool:
        """验证 Spec Kit 是否已正确安装

        检查 specify 命令是否可用，并尝试执行 specify version 命令。

        Returns:
            bool: 如果验证成功返回 True，否则返回 False
        """
        # 检查 specify 命令是否在 PATH 中
        if not shutil.which("specify"):
            return False

        # 尝试执行 specify version 命令
        returncode, stdout, stderr = self.run_command(
            "specify version",
            shell=True,
            timeout=10,
            max_retries=1
        )

        return returncode == 0
    
    def _get_installed_version(self) -> Optional[str]:
        """获取已安装的 Spec Kit 版本

        从 specify version 输出中提取 CLI Version。

        Returns:
            Optional[str]: 版本号，如果无法获取则返回 None
        """
        if not shutil.which("specify"):
            return None

        returncode, stdout, stderr = self.run_command(
            "specify version",
            shell=True,
            timeout=10,
            max_retries=1
        )

        if returncode == 0:
            # specify version 输出富文本表格，提取 "CLI Version    0.0.22"
            match = re.search(r"CLI Version\s+(\S+)", stdout)
            if match:
                return match.group(1)

        return None
    
    def install(self) -> InstallReport:
        """安装 Spec Kit
        
        执行以下步骤：
        1. 检查 Spec Kit 是否已安装
        2. 检查 uv 是否可用
        3. 使用 uv tool install 从 GitHub 安装
        4. 验证安装
        
        Returns:
            InstallReport: 安装报告
        """
        # 检查是否已安装
        if self.verify():
            version = self._get_installed_version()
            return InstallReport(
                tool_name="spec-kit",
                result=InstallResult.SKIPPED,
                message="Spec Kit 已安装，跳过安装",
                version=version
            )
        
        try:
            # 检查 uv 是否可用
            if not shutil.which("uv"):
                return InstallReport(
                    tool_name="spec-kit",
                    result=InstallResult.FAILED,
                    message="uv 未安装，无法安装 Spec Kit",
                    error="请先安装 uv"
                )
            
            # 使用 uv tool install 从 GitHub 安装
            install_cmd = f"uv tool install specify-cli --from {self.github_repo}"
            returncode, stdout, stderr = self.run_command(
                install_cmd,
                shell=True,
                timeout=300,
                max_retries=2
            )
            
            if returncode != 0:
                return InstallReport(
                    tool_name="spec-kit",
                    result=InstallResult.FAILED,
                    message="使用 uv tool install 安装 Spec Kit 失败",
                    error=stderr or "安装命令返回非零退出码"
                )
            
            # 验证安装
            if not self.verify():
                return InstallReport(
                    tool_name="spec-kit",
                    result=InstallResult.FAILED,
                    message="Spec Kit 安装验证失败",
                    error="安装命令执行成功，但无法验证 Spec Kit 是否正确安装"
                )
            
            # 获取安装的版本
            version = self._get_installed_version()
            
            return InstallReport(
                tool_name="spec-kit",
                result=InstallResult.SUCCESS,
                message="Spec Kit 安装成功",
                version=version
            )
            
        except Exception as e:
            return InstallReport(
                tool_name="spec-kit",
                result=InstallResult.FAILED,
                message="Spec Kit 安装过程中发生异常",
                error=str(e)
            )
    
    def upgrade(self) -> InstallReport:
        """升级 Spec Kit
        
        Spec Kit 的升级通过执行 `uv tool install specify-cli --force --from <github_repo>` 命令实现。
        
        Returns:
            InstallReport: 升级报告
        """
        # 检查是否已安装
        if not self.verify():
            return InstallReport(
                tool_name="spec-kit",
                result=InstallResult.FAILED,
                message="Spec Kit 未安装，无法升级",
                error="请先安装 Spec Kit"
            )
        
        try:
            # 检查 uv 是否可用
            if not shutil.which("uv"):
                return InstallReport(
                    tool_name="spec-kit",
                    result=InstallResult.FAILED,
                    message="uv 未安装，无法升级 Spec Kit",
                    error="请先安装 uv"
                )
            
            # 记录升级前的版本
            old_version = self._get_installed_version()
            
            # 执行升级命令（使用 --force 强制重新安装）
            upgrade_cmd = f"uv tool install specify-cli --force --from {self.github_repo}"
            returncode, stdout, stderr = self.run_command(
                upgrade_cmd,
                shell=True,
                timeout=300,
                max_retries=2
            )
            
            if returncode != 0:
                return InstallReport(
                    tool_name="spec-kit",
                    result=InstallResult.FAILED,
                    message="执行 Spec Kit 升级命令失败",
                    error=stderr or "升级命令返回非零退出码"
                )
            
            # 验证升级
            if not self.verify():
                return InstallReport(
                    tool_name="spec-kit",
                    result=InstallResult.FAILED,
                    message="Spec Kit 升级验证失败",
                    error="升级命令执行成功，但无法验证 Spec Kit 是否正确安装"
                )
            
            # 获取升级后的版本
            new_version = self._get_installed_version()
            
            return InstallReport(
                tool_name="spec-kit",
                result=InstallResult.SUCCESS,
                message=f"Spec Kit 升级成功 ({old_version} -> {new_version})",
                version=new_version
            )
            
        except Exception as e:
            return InstallReport(
                tool_name="spec-kit",
                result=InstallResult.FAILED,
                message="Spec Kit 升级过程中发生异常",
                error=str(e)
            )



"""配置文件加载集成测试"""

import pytest
from pathlib import Path
import yaml

from mono_kickstart.config import ConfigManager, Config, ProjectConfig, ToolConfig, RegistryConfig


class TestConfigLoadingIntegration:
    """配置文件加载集成测试"""
    
    def test_load_single_config_file(self, tmp_path):
        """测试加载单个配置文件"""
        # 创建配置文件
        config_file = tmp_path / ".kickstartrc"
        config_data = {
            "project": {
                "name": "test-project"
            },
            "tools": {
                "nvm": {"enabled": True},
                "node": {"enabled": True, "version": "lts"}
            },
            "registry": {
                "npm": "https://registry.npmmirror.com/"
            }
        }
        
        with open(config_file, 'w') as f:
            yaml.dump(config_data, f)
        
        # 加载配置
        config_manager = ConfigManager()
        config = config_manager.load_from_file(config_file)
        
        # 验证配置
        assert config.project.name == "test-project"
        assert config.tools["nvm"].enabled is True
        assert config.tools["node"].enabled is True
        assert config.tools["node"].version == "lts"
        assert config.registry.npm == "https://registry.npmmirror.com/"
    
    def test_load_with_priority(self, tmp_path):
        """测试按优先级加载多个配置文件"""
        # 创建用户配置文件
        user_config = tmp_path / "user.yaml"
        user_config_data = {
            "project": {"name": "user-project"},
            "tools": {
                "nvm": {"enabled": True},
                "node": {"enabled": True, "version": "18.17.0"}
            },
            "registry": {
                "npm": "https://registry.npmjs.org/",
                "pypi": "https://pypi.org/simple"
            }
        }
        with open(user_config, 'w') as f:
            yaml.dump(user_config_data, f)
        
        # 创建项目配置文件
        project_config = tmp_path / "project.yaml"
        project_config_data = {
            "project": {"name": "my-project"},
            "tools": {
                "node": {"version": "lts"},
                "bun": {"enabled": True}
            }
        }
        with open(project_config, 'w') as f:
            yaml.dump(project_config_data, f)
        
        # 加载配置
        config_manager = ConfigManager()
        
        # 按优先级加载：项目配置 > 用户配置 > 默认配置
        config = config_manager.load_with_priority(
            cli_config=None,
            project_config=project_config,
            user_config=user_config
        )
        
        # 验证合并结果
        # 项目配置覆盖用户配置
        assert config.project.name == "my-project"
        assert config.tools["node"].version == "lts"
        
        # 用户配置保留（项目配置未指定）
        assert config.tools["nvm"].enabled is True
        assert config.registry.pypi == "https://pypi.org/simple"
        
        # 项目配置新增
        assert config.tools["bun"].enabled is True
    
    def test_load_with_cli_config_priority(self, tmp_path):
        """测试 CLI 配置优先级最高"""
        # 创建用户配置
        user_config = tmp_path / "user.yaml"
        user_config_data = {
            "project": {"name": "user-project"},
            "tools": {"node": {"version": "18.17.0"}}
        }
        with open(user_config, 'w') as f:
            yaml.dump(user_config_data, f)
        
        # 创建项目配置
        project_config = tmp_path / "project.yaml"
        project_config_data = {
            "project": {"name": "project-name"},
            "tools": {"node": {"version": "lts"}}
        }
        with open(project_config, 'w') as f:
            yaml.dump(project_config_data, f)
        
        # 创建 CLI 配置
        cli_config = tmp_path / "cli.yaml"
        cli_config_data = {
            "project": {"name": "cli-project"},
            "tools": {"node": {"version": "20.0.0"}}
        }
        with open(cli_config, 'w') as f:
            yaml.dump(cli_config_data, f)
        
        # 加载配置
        config_manager = ConfigManager()
        config = config_manager.load_with_priority(
            cli_config=cli_config,
            project_config=project_config,
            user_config=user_config
        )
        
        # 验证 CLI 配置优先级最高
        assert config.project.name == "cli-project"
        assert config.tools["node"].version == "20.0.0"
    
    def test_load_with_missing_files(self, tmp_path):
        """测试加载不存在的配置文件"""
        # 创建一个存在的配置文件
        existing_config = tmp_path / "existing.yaml"
        existing_config_data = {
            "project": {"name": "test-project"}
        }
        with open(existing_config, 'w') as f:
            yaml.dump(existing_config_data, f)
        
        # 不存在的配置文件路径
        missing_config = tmp_path / "missing.yaml"
        
        # 加载配置
        config_manager = ConfigManager()
        config = config_manager.load_with_priority(
            cli_config=None,
            project_config=existing_config,
            user_config=missing_config  # 不存在
        )
        
        # 验证能正常加载存在的配置
        assert config.project.name == "test-project"
    
    def test_save_and_reload_config(self, tmp_path):
        """测试保存和重新加载配置"""
        # 创建配置
        config = Config()
        config.project = ProjectConfig(name="test-project")
        config.tools["nvm"] = ToolConfig(enabled=True)
        config.tools["node"] = ToolConfig(enabled=True, version="lts")
        config.registry = RegistryConfig(
            npm="https://registry.npmmirror.com/",
            pypi="https://mirrors.sustech.edu.cn/pypi/web/simple"
        )
        
        # 保存配置
        config_file = tmp_path / ".kickstartrc"
        config_manager = ConfigManager()
        config_manager.save_to_file(config, config_file)
        
        # 验证文件存在
        assert config_file.exists()
        
        # 重新加载配置
        loaded_config = config_manager.load_from_file(config_file)
        
        # 验证配置一致
        assert loaded_config.project.name == config.project.name
        assert loaded_config.tools["nvm"].enabled == config.tools["nvm"].enabled
        assert loaded_config.tools["node"].version == config.tools["node"].version
        assert loaded_config.registry.npm == config.registry.npm
        assert loaded_config.registry.pypi == config.registry.pypi
    
    def test_config_roundtrip_consistency(self, tmp_path):
        """测试配置往返一致性"""
        # 创建复杂配置
        config = Config()
        config.project = ProjectConfig(name="complex-project")
        
        # 配置多个工具
        config.tools["nvm"] = ToolConfig(enabled=True)
        config.tools["node"] = ToolConfig(enabled=True, version="18.17.0")
        config.tools["conda"] = ToolConfig(enabled=False)
        config.tools["bun"] = ToolConfig(enabled=True, version="latest")
        config.tools["uv"] = ToolConfig(enabled=True)
        
        # 配置镜像源
        config.registry = RegistryConfig(
            npm="https://registry.npmmirror.com/",
            bun="https://registry.npmmirror.com/",
            pypi="https://mirrors.sustech.edu.cn/pypi/web/simple",
            python_install="https://ghfast.top/https://github.com/astral-sh/python-build-standalone/releases/download"
        )
        
        # 保存配置
        config_file = tmp_path / ".kickstartrc"
        config_manager = ConfigManager()
        config_manager.save_to_file(config, config_file)
        
        # 重新加载
        loaded_config = config_manager.load_from_file(config_file)
        
        # 验证所有字段一致
        assert loaded_config.project.name == config.project.name
        
        for tool_name in ["nvm", "node", "conda", "bun", "uv"]:
            assert loaded_config.tools[tool_name].enabled == config.tools[tool_name].enabled
            assert loaded_config.tools[tool_name].version == config.tools[tool_name].version
        
        assert loaded_config.registry.npm == config.registry.npm
        assert loaded_config.registry.bun == config.registry.bun
        assert loaded_config.registry.pypi == config.registry.pypi
        assert loaded_config.registry.python_install == config.registry.python_install
    
    def test_merge_nested_configs(self, tmp_path):
        """测试嵌套配置的合并"""
        # 创建基础配置
        base_config = tmp_path / "base.yaml"
        base_config_data = {
            "tools": {
                "nvm": {"enabled": True},
                "node": {"enabled": True, "version": "18.17.0"},
                "bun": {"enabled": True}
            }
        }
        with open(base_config, 'w') as f:
            yaml.dump(base_config_data, f)
        
        # 创建覆盖配置
        override_config = tmp_path / "override.yaml"
        override_config_data = {
            "tools": {
                "node": {"version": "lts"},  # 覆盖版本
                "bun": {"enabled": False},  # 禁用 bun
                "uv": {"enabled": True}  # 新增 uv
            }
        }
        with open(override_config, 'w') as f:
            yaml.dump(override_config_data, f)
        
        # 加载并合并配置
        config_manager = ConfigManager()
        base = config_manager.load_from_file(base_config)
        override = config_manager.load_from_file(override_config)
        merged = config_manager.merge_configs(base, override)
        
        # 验证合并结果
        assert merged.tools["nvm"].enabled is True  # 保留基础配置
        assert merged.tools["node"].enabled is True  # 保留基础配置
        assert merged.tools["node"].version == "lts"  # 覆盖版本
        assert merged.tools["bun"].enabled is False  # 覆盖启用状态
        assert merged.tools["uv"].enabled is True  # 新增工具
    
    def test_config_validation(self, tmp_path):
        """测试配置验证"""
        # 创建有效配置
        valid_config = Config()
        valid_config.project = ProjectConfig(name="valid-project")
        valid_config.tools["nvm"] = ToolConfig(enabled=True)
        
        config_manager = ConfigManager()
        errors = config_manager.validate(valid_config)
        
        # 验证有效配置无错误
        assert len(errors) == 0
    
    def test_load_config_with_extra_fields(self, tmp_path):
        """测试加载包含额外字段的配置文件"""
        # 创建包含额外字段的配置文件
        config_file = tmp_path / ".kickstartrc"
        config_data = {
            "project": {
                "name": "test-project"
                # 注意：dataclass 不支持额外字段，所以不测试额外字段
            },
            "tools": {
                "nvm": {"enabled": True}
            }
            # 注意：未知部分会被忽略
        }
        
        with open(config_file, 'w') as f:
            yaml.dump(config_data, f)
        
        # 加载配置
        config_manager = ConfigManager()
        config = config_manager.load_from_file(config_file)
        
        # 验证已知字段正常加载
        assert config.project.name == "test-project"
        assert config.tools["nvm"].enabled is True



"""
Unit tests for installer base class module

Tests command execution, file download, retry logic, timeout handling,
and network error scenarios for the ToolInstaller base class.

**Validates: Requirements 1.12**
"""

import subprocess
import time
import urllib.error
import urllib.request
from pathlib import Path
from unittest.mock import MagicMock, Mock, call, mock_open, patch

import pytest

from mono_kickstart.config import ToolConfig
from mono_kickstart.installer_base import (
    InstallReport,
    InstallResult,
    ToolInstaller,
)
from mono_kickstart.platform_detector import Arch, OS, PlatformInfo, Shell


# Concrete implementation for testing abstract base class
class ConcreteInstaller(ToolInstaller):
    """Concrete installer implementation for testing"""
    
    def install(self) -> InstallReport:
        return InstallReport(
            tool_name="test-tool",
            result=InstallResult.SUCCESS,
            message="Installation successful"
        )
    
    def upgrade(self) -> InstallReport:
        return InstallReport(
            tool_name="test-tool",
            result=InstallResult.SUCCESS,
            message="Upgrade successful"
        )
    
    def verify(self) -> bool:
        return True


@pytest.fixture
def platform_info():
    """Create a PlatformInfo instance for testing"""
    return PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file="~/.bashrc"
    )


@pytest.fixture
def tool_config():
    """Create a ToolConfig instance for testing"""
    return ToolConfig(enabled=True, version=None)


@pytest.fixture
def installer(platform_info, tool_config):
    """Create a ConcreteInstaller instance for testing"""
    return ConcreteInstaller(platform_info, tool_config)


class TestInstallResultEnum:
    """Test InstallResult enumeration"""
    
    def test_install_result_values(self):
        """Test InstallResult enum has correct values"""
        assert InstallResult.SUCCESS.value == "success"
        assert InstallResult.SKIPPED.value == "skipped"
        assert InstallResult.FAILED.value == "failed"
    
    def test_install_result_members(self):
        """Test InstallResult enum has all expected members"""
        members = [member.name for member in InstallResult]
        assert "SUCCESS" in members
        assert "SKIPPED" in members
        assert "FAILED" in members
        assert len(members) == 3


class TestInstallReportDataClass:
    """Test InstallReport data class"""
    
    def test_install_report_creation_with_all_fields(self):
        """Test creating InstallReport with all fields"""
        report = InstallReport(
            tool_name="node",
            result=InstallResult.SUCCESS,
            message="Installation successful",
            version="20.11.0",
            error=None
        )
        
        assert report.tool_name == "node"
        assert report.result == InstallResult.SUCCESS
        assert report.message == "Installation successful"
        assert report.version == "20.11.0"
        assert report.error is None
    
    def test_install_report_creation_with_minimal_fields(self):
        """Test creating InstallReport with only required fields"""
        report = InstallReport(
            tool_name="bun",
            result=InstallResult.FAILED,
            message="Installation failed"
        )
        
        assert report.tool_name == "bun"
        assert report.result == InstallResult.FAILED
        assert report.message == "Installation failed"
        assert report.version is None
        assert report.error is None
    
    def test_install_report_with_error(self):
        """Test creating InstallReport with error information"""
        report = InstallReport(
            tool_name="uv",
            result=InstallResult.FAILED,
            message="Download failed",
            error="Network timeout"
        )
        
        assert report.tool_name == "uv"
        assert report.result == InstallResult.FAILED
        assert report.error == "Network timeout"


class TestToolInstallerInitialization:
    """Test ToolInstaller initialization"""
    
    def test_installer_initialization(self, platform_info, tool_config):
        """Test installer initializes with correct attributes"""
        installer = ConcreteInstaller(platform_info, tool_config)
        
        assert installer.platform_info == platform_info
        assert installer.config == tool_config
    
    def test_installer_stores_platform_info(self, platform_info, tool_config):
        """Test installer stores platform information correctly"""
        installer = ConcreteInstaller(platform_info, tool_config)
        
        assert installer.platform_info.os == OS.LINUX
        assert installer.platform_info.arch == Arch.X86_64
        assert installer.platform_info.shell == Shell.BASH
    
    def test_installer_stores_config(self, platform_info, tool_config):
        """Test installer stores configuration correctly"""
        tool_config.enabled = False
        tool_config.version = "1.2.3"
        installer = ConcreteInstaller(platform_info, tool_config)
        
        assert installer.config.enabled is False
        assert installer.config.version == "1.2.3"


class TestRunCommandSuccess:
    """Test run_command method with successful execution"""
    
    def test_run_command_success(self, installer):
        """Test run_command returns correct output on success"""
        mock_result = Mock()
        mock_result.returncode = 0
        mock_result.stdout = "command output"
        mock_result.stderr = ""
        
        with patch('subprocess.run', return_value=mock_result):
            returncode, stdout, stderr = installer.run_command("echo test")
            
            assert returncode == 0
            assert stdout == "command output"
            assert stderr == ""
    
    def test_run_command_with_stderr(self, installer):
        """Test run_command captures stderr output"""
        mock_result = Mock()
        mock_result.returncode = 0
        mock_result.stdout = "output"
        mock_result.stderr = "warning message"
        
        with patch('subprocess.run', return_value=mock_result):
            returncode, stdout, stderr = installer.run_command("command")
            
            assert returncode == 0
            assert stdout == "output"
            assert stderr == "warning message"
    
    def test_run_command_with_nonzero_exit_code(self, installer):
        """Test run_command handles non-zero exit codes"""
        mock_result = Mock()
        mock_result.returncode = 1
        mock_result.stdout = ""
        mock_result.stderr = "error message"
        
        with patch('subprocess.run', return_value=mock_result):
            returncode, stdout, stderr = installer.run_command("failing-command")
            
            assert returncode == 1
            assert stderr == "error message"
    
    def test_run_command_with_shell_parameter(self, installer):
        """Test run_command respects shell parameter"""
        mock_result = Mock()
        mock_result.returncode = 0
        mock_result.stdout = "output"
        mock_result.stderr = ""
        
        with patch('subprocess.run', return_value=mock_result) as mock_run:
            installer.run_command("command", shell=False)
            
            mock_run.assert_called_once()
            call_kwargs = mock_run.call_args[1]
            assert call_kwargs['shell'] is False
    
    def test_run_command_with_custom_timeout(self, installer):
        """Test run_command respects custom timeout"""
        mock_result = Mock()
        mock_result.returncode = 0
        mock_result.stdout = "output"
        mock_result.stderr = ""
        
        with patch('subprocess.run', return_value=mock_result) as mock_run:
            installer.run_command("command", timeout=60)
            
            mock_run.assert_called_once()
            call_kwargs = mock_run.call_args[1]
            assert call_kwargs['timeout'] == 60


class TestRunCommandRetry:
    """Test run_command retry logic"""
    
    def test_run_command_retries_on_subprocess_error(self, installer):
        """Test run_command retries on subprocess error"""
        mock_result = Mock()
        mock_result.returncode = 0
        mock_result.stdout = "success"
        mock_result.stderr = ""
        
        with patch('subprocess.run', side_effect=[
            subprocess.SubprocessError("error 1"),
            subprocess.SubprocessError("error 2"),
            mock_result
        ]) as mock_run:
            with patch('time.sleep'):
                returncode, stdout, stderr = installer.run_command("command", max_retries=3)
                
                assert returncode == 0
                assert stdout == "success"
                assert mock_run.call_count == 3
    
    def test_run_command_fails_after_max_retries(self, installer):
        """Test run_command fails after exhausting retries"""
        with patch('subprocess.run', side_effect=subprocess.SubprocessError("persistent error")):
            with patch('time.sleep'):
                returncode, stdout, stderr = installer.run_command("command", max_retries=3)
                
                assert returncode == -1
                assert stdout == ""
                assert "命令执行失败" in stderr
                assert "persistent error" in stderr
    
    def test_run_command_exponential_backoff(self, installer):
        """Test run_command uses exponential backoff for retries"""
        with patch('subprocess.run', side_effect=subprocess.SubprocessError("error")):
            with patch('time.sleep') as mock_sleep:
                installer.run_command("command", max_retries=3, retry_delay=1)
                
                # Should sleep twice (after first and second attempts)
                assert mock_sleep.call_count == 2
                # First retry: 1 second, second retry: 2 seconds
                mock_sleep.assert_has_calls([call(1), call(2)])
    
    def test_run_command_custom_retry_delay(self, installer):
        """Test run_command respects custom retry delay"""
        with patch('subprocess.run', side_effect=subprocess.SubprocessError("error")):
            with patch('time.sleep') as mock_sleep:
                installer.run_command("command", max_retries=3, retry_delay=2)
                
                # First retry: 2 seconds, second retry: 4 seconds
                mock_sleep.assert_has_calls([call(2), call(4)])
    
    def test_run_command_no_retry_on_success(self, installer):
        """Test run_command doesn't retry on first success"""
        mock_result = Mock()
        mock_result.returncode = 0
        mock_result.stdout = "success"
        mock_result.stderr = ""
        
        with patch('subprocess.run', return_value=mock_result) as mock_run:
            with patch('time.sleep') as mock_sleep:
                installer.run_command("command", max_retries=3)
                
                assert mock_run.call_count == 1
                assert mock_sleep.call_count == 0


class TestRunCommandTimeout:
    """Test run_command timeout handling"""
    
    def test_run_command_timeout_raises_exception(self, installer):
        """Test run_command raises TimeoutExpired on timeout"""
        with patch('subprocess.run', side_effect=subprocess.TimeoutExpired('cmd', 5)):
            with pytest.raises(subprocess.TimeoutExpired):
                installer.run_command("slow-command", timeout=5)
    
    def test_run_command_timeout_no_retry(self, installer):
        """Test run_command doesn't retry on timeout"""
        with patch('subprocess.run', side_effect=subprocess.TimeoutExpired('cmd', 5)) as mock_run:
            with patch('time.sleep') as mock_sleep:
                with pytest.raises(subprocess.TimeoutExpired):
                    installer.run_command("slow-command", max_retries=3, timeout=5)
                
                # Should only try once, no retries on timeout
                assert mock_run.call_count == 1
                assert mock_sleep.call_count == 0
    
    def test_run_command_with_zero_timeout(self, installer):
        """Test run_command with zero timeout"""
        mock_result = Mock()
        mock_result.returncode = 0
        mock_result.stdout = "output"
        mock_result.stderr = ""
        
        with patch('subprocess.run', return_value=mock_result) as mock_run:
            installer.run_command("command", timeout=0)
            
            call_kwargs = mock_run.call_args[1]
            assert call_kwargs['timeout'] == 0


class TestDownloadFileSuccess:
    """Test download_file method with successful downloads"""
    
    def test_download_file_success(self, installer, tmp_path):
        """Test download_file successfully downloads a file"""
        dest_file = tmp_path / "test.txt"
        file_content = b"test content"
        
        mock_response = Mock()
        mock_response.read.return_value = file_content
        mock_response.__enter__ = Mock(return_value=mock_response)
        mock_response.__exit__ = Mock(return_value=False)
        
        with patch('urllib.request.urlopen', return_value=mock_response):
            result = installer.download_file("http://example.com/file.txt", str(dest_file))
            
            assert result is True
            assert dest_file.exists()
            assert dest_file.read_bytes() == file_content
    
    def test_download_file_creates_parent_directories(self, installer, tmp_path):
        """Test download_file creates parent directories if they don't exist"""
        dest_file = tmp_path / "subdir" / "nested" / "test.txt"
        file_content = b"content"
        
        mock_response = Mock()
        mock_response.read.return_value = file_content
        mock_response.__enter__ = Mock(return_value=mock_response)
        mock_response.__exit__ = Mock(return_value=False)
        
        with patch('urllib.request.urlopen', return_value=mock_response):
            result = installer.download_file("http://example.com/file.txt", str(dest_file))
            
            assert result is True
            assert dest_file.exists()
            assert dest_file.parent.exists()
    
    def test_download_file_verifies_file_exists(self, installer, tmp_path):
        """Test download_file verifies the downloaded file exists"""
        dest_file = tmp_path / "test.txt"
        
        mock_response = Mock()
        mock_response.read.return_value = b"content"
        mock_response.__enter__ = Mock(return_value=mock_response)
        mock_response.__exit__ = Mock(return_value=False)
        
        with patch('urllib.request.urlopen', return_value=mock_response):
            result = installer.download_file("http://example.com/file.txt", str(dest_file))
            
            assert result is True
            assert dest_file.exists()
    
    def test_download_file_verifies_file_not_empty(self, installer, tmp_path):
        """Test download_file verifies the downloaded file is not empty"""
        dest_file = tmp_path / "test.txt"
        
        # Create empty file
        mock_response = Mock()
        mock_response.read.return_value = b""
        mock_response.__enter__ = Mock(return_value=mock_response)
        mock_response.__exit__ = Mock(return_value=False)
        
        with patch('urllib.request.urlopen', return_value=mock_response):
            result = installer.download_file("http://example.com/file.txt", str(dest_file))
            
            # Should fail because file is empty
            assert result is False


class TestDownloadFileRetry:
    """Test download_file retry logic"""
    
    def test_download_file_retries_on_url_error(self, installer, tmp_path):
        """Test download_file retries on URLError"""
        dest_file = tmp_path / "test.txt"
        file_content = b"content"
        
        mock_response = Mock()
        mock_response.read.return_value = file_content
        mock_response.__enter__ = Mock(return_value=mock_response)
        mock_response.__exit__ = Mock(return_value=False)
        
        with patch('urllib.request.urlopen', side_effect=[
            urllib.error.URLError("error 1"),
            urllib.error.URLError("error 2"),
            mock_response
        ]) as mock_urlopen:
            with patch('time.sleep'):
                result = installer.download_file("http://example.com/file.txt", str(dest_file), max_retries=3)
                
                assert result is True
                assert mock_urlopen.call_count == 3
    
    def test_download_file_retries_on_http_error(self, installer, tmp_path):
        """Test download_file retries on HTTPError"""
        dest_file = tmp_path / "test.txt"
        file_content = b"content"
        
        mock_response = Mock()
        mock_response.read.return_value = file_content
        mock_response.__enter__ = Mock(return_value=mock_response)
        mock_response.__exit__ = Mock(return_value=False)
        
        with patch('urllib.request.urlopen', side_effect=[
            urllib.error.HTTPError("url", 500, "Server Error", {}, None),
            mock_response
        ]) as mock_urlopen:
            with patch('time.sleep'):
                result = installer.download_file("http://example.com/file.txt", str(dest_file), max_retries=3)
                
                assert result is True
                assert mock_urlopen.call_count == 2
    
    def test_download_file_fails_after_max_retries(self, installer, tmp_path):
        """Test download_file fails after exhausting retries"""
        dest_file = tmp_path / "test.txt"
        
        with patch('urllib.request.urlopen', side_effect=urllib.error.URLError("persistent error")):
            with patch('time.sleep'):
                result = installer.download_file("http://example.com/file.txt", str(dest_file), max_retries=3)
                
                assert result is False
                # File should not exist after failed download
                assert not dest_file.exists()
    
    def test_download_file_exponential_backoff(self, installer, tmp_path):
        """Test download_file uses exponential backoff for retries"""
        dest_file = tmp_path / "test.txt"
        
        with patch('urllib.request.urlopen', side_effect=urllib.error.URLError("error")):
            with patch('time.sleep') as mock_sleep:
                installer.download_file("http://example.com/file.txt", str(dest_file), max_retries=3, retry_delay=1)
                
                # Should sleep twice (after first and second attempts)
                assert mock_sleep.call_count == 2
                # First retry: 1 second, second retry: 2 seconds
                mock_sleep.assert_has_calls([call(1), call(2)])
    
    def test_download_file_custom_retry_delay(self, installer, tmp_path):
        """Test download_file respects custom retry delay"""
        dest_file = tmp_path / "test.txt"
        
        with patch('urllib.request.urlopen', side_effect=urllib.error.URLError("error")):
            with patch('time.sleep') as mock_sleep:
                installer.download_file("http://example.com/file.txt", str(dest_file), max_retries=3, retry_delay=2)
                
                # First retry: 2 seconds, second retry: 4 seconds
                mock_sleep.assert_has_calls([call(2), call(4)])
    
    def test_download_file_deletes_partial_download_on_error(self, installer, tmp_path):
        """Test download_file deletes partially downloaded file on error"""
        dest_file = tmp_path / "test.txt"
        
        # Create a partial file
        dest_file.write_bytes(b"partial")
        
        with patch('urllib.request.urlopen', side_effect=urllib.error.URLError("error")):
            with patch('time.sleep'):
                result = installer.download_file("http://example.com/file.txt", str(dest_file), max_retries=1)
                
                assert result is False
                # Partial file should be deleted
                assert not dest_file.exists()


class TestDownloadFileNetworkErrors:
    """Test download_file network error handling"""
    
    def test_download_file_handles_connection_error(self, installer, tmp_path):
        """Test download_file handles connection errors"""
        dest_file = tmp_path / "test.txt"
        
        with patch('urllib.request.urlopen', side_effect=urllib.error.URLError("Connection refused")):
            with patch('time.sleep'):
                result = installer.download_file("http://example.com/file.txt", str(dest_file), max_retries=2)
                
                assert result is False
    
    def test_download_file_handles_404_error(self, installer, tmp_path):
        """Test download_file handles 404 errors"""
        dest_file = tmp_path / "test.txt"
        
        with patch('urllib.request.urlopen', side_effect=urllib.error.HTTPError("url", 404, "Not Found", {}, None)):
            with patch('time.sleep'):
                result = installer.download_file("http://example.com/file.txt", str(dest_file), max_retries=2)
                
                assert result is False
    
    def test_download_file_handles_500_error(self, installer, tmp_path):
        """Test download_file handles 500 errors"""
        dest_file = tmp_path / "test.txt"
        
        with patch('urllib.request.urlopen', side_effect=urllib.error.HTTPError("url", 500, "Server Error", {}, None)):
            with patch('time.sleep'):
                result = installer.download_file("http://example.com/file.txt", str(dest_file), max_retries=2)
                
                assert result is False
    
    def test_download_file_handles_timeout(self, installer, tmp_path):
        """Test download_file handles timeout errors"""
        dest_file = tmp_path / "test.txt"
        
        with patch('urllib.request.urlopen', side_effect=urllib.error.URLError("timeout")):
            with patch('time.sleep'):
                result = installer.download_file("http://example.com/file.txt", str(dest_file), max_retries=2, timeout=10)
                
                assert result is False
    
    def test_download_file_with_custom_timeout(self, installer, tmp_path):
        """Test download_file respects custom timeout"""
        dest_file = tmp_path / "test.txt"
        file_content = b"content"
        
        mock_response = Mock()
        mock_response.read.return_value = file_content
        mock_response.__enter__ = Mock(return_value=mock_response)
        mock_response.__exit__ = Mock(return_value=False)
        
        with patch('urllib.request.urlopen', return_value=mock_response) as mock_urlopen:
            installer.download_file("http://example.com/file.txt", str(dest_file), timeout=60)
            
            mock_urlopen.assert_called_once()
            call_args = mock_urlopen.call_args
            assert call_args[1]['timeout'] == 60
    
    def test_download_file_handles_os_error(self, installer, tmp_path):
        """Test download_file handles OS errors (e.g., permission denied)"""
        dest_file = tmp_path / "test.txt"
        
        mock_response = Mock()
        mock_response.read.return_value = b"content"
        mock_response.__enter__ = Mock(return_value=mock_response)
        mock_response.__exit__ = Mock(return_value=False)
        
        with patch('urllib.request.urlopen', return_value=mock_response):
            with patch('builtins.open', side_effect=OSError("Permission denied")):
                with patch('time.sleep'):
                    result = installer.download_file("http://example.com/file.txt", str(dest_file), max_retries=2)
                    
                    assert result is False


class TestAbstractMethods:
    """Test abstract method enforcement"""
    
    def test_concrete_installer_implements_install(self, installer):
        """Test concrete installer implements install method"""
        report = installer.install()
        
        assert isinstance(report, InstallReport)
        assert report.tool_name == "test-tool"
        assert report.result == InstallResult.SUCCESS
    
    def test_concrete_installer_implements_upgrade(self, installer):
        """Test concrete installer implements upgrade method"""
        report = installer.upgrade()
        
        assert isinstance(report, InstallReport)
        assert report.tool_name == "test-tool"
        assert report.result == InstallResult.SUCCESS
    
    def test_concrete_installer_implements_verify(self, installer):
        """Test concrete installer implements verify method"""
        result = installer.verify()
        
        assert isinstance(result, bool)
        assert result is True
    
    def test_cannot_instantiate_abstract_base_class(self, platform_info, tool_config):
        """Test ToolInstaller cannot be instantiated directly"""
        with pytest.raises(TypeError):
            ToolInstaller(platform_info, tool_config)


class TestEdgeCases:
    """Test edge cases and boundary conditions"""
    
    def test_run_command_with_empty_command(self, installer):
        """Test run_command with empty command string"""
        mock_result = Mock()
        mock_result.returncode = 0
        mock_result.stdout = ""
        mock_result.stderr = ""
        
        with patch('subprocess.run', return_value=mock_result):
            returncode, stdout, stderr = installer.run_command("")
            
            assert returncode == 0
    
    def test_run_command_with_max_retries_zero(self, installer):
        """Test run_command with max_retries=0 (should still try once)"""
        mock_result = Mock()
        mock_result.returncode = 0
        mock_result.stdout = "output"
        mock_result.stderr = ""
        
        with patch('subprocess.run', return_value=mock_result) as mock_run:
            # max_retries=0 should still execute once
            returncode, stdout, stderr = installer.run_command("command", max_retries=1)
            
            assert mock_run.call_count == 1
    
    def test_download_file_with_max_retries_zero(self, installer, tmp_path):
        """Test download_file with max_retries=0 (should still try once)"""
        dest_file = tmp_path / "test.txt"
        file_content = b"content"
        
        mock_response = Mock()
        mock_response.read.return_value = file_content
        mock_response.__enter__ = Mock(return_value=mock_response)
        mock_response.__exit__ = Mock(return_value=False)
        
        with patch('urllib.request.urlopen', return_value=mock_response) as mock_urlopen:
            result = installer.download_file("http://example.com/file.txt", str(dest_file), max_retries=1)
            
            assert result is True
            assert mock_urlopen.call_count == 1
    
    def test_download_file_with_special_characters_in_path(self, installer, tmp_path):
        """Test download_file with special characters in file path"""
        dest_file = tmp_path / "test file (1).txt"
        file_content = b"content"
        
        mock_response = Mock()
        mock_response.read.return_value = file_content
        mock_response.__enter__ = Mock(return_value=mock_response)
        mock_response.__exit__ = Mock(return_value=False)
        
        with patch('urllib.request.urlopen', return_value=mock_response):
            result = installer.download_file("http://example.com/file.txt", str(dest_file))
            
            assert result is True
            assert dest_file.exists()
    
    def test_run_command_with_very_long_output(self, installer):
        """Test run_command handles very long output"""
        long_output = "x" * 100000  # 100KB of output
        
        mock_result = Mock()
        mock_result.returncode = 0
        mock_result.stdout = long_output
        mock_result.stderr = ""
        
        with patch('subprocess.run', return_value=mock_result):
            returncode, stdout, stderr = installer.run_command("command")
            
            assert returncode == 0
            assert len(stdout) == 100000
    
    def test_download_file_with_large_file(self, installer, tmp_path):
        """Test download_file handles large files"""
        dest_file = tmp_path / "large.bin"
        large_content = b"x" * (10 * 1024 * 1024)  # 10MB
        
        mock_response = Mock()
        mock_response.read.return_value = large_content
        mock_response.__enter__ = Mock(return_value=mock_response)
        mock_response.__exit__ = Mock(return_value=False)
        
        with patch('urllib.request.urlopen', return_value=mock_response):
            result = installer.download_file("http://example.com/large.bin", str(dest_file))
            
            assert result is True
            assert dest_file.stat().st_size == 10 * 1024 * 1024



"""安装编排器模块

该模块负责编排整个安装流程，包括：
- 确定工具安装顺序（基于依赖关系）
- 执行单个工具的安装逻辑（检测、跳过或安装）
- 执行批量工具安装（按顺序，容错）
- 调用镜像源配置
- 调用项目创建
- 生成和打印安装摘要
- 执行升级流程（全部或单个工具）
"""

from pathlib import Path
from typing import Dict, List, Optional

from .config import Config, ToolConfig
from .installer_base import InstallReport, InstallResult, ToolInstaller
from .installers.bmad_installer import BMadInstaller
from .installers.bun_installer import BunInstaller
from .installers.claude_installer import ClaudeCodeInstaller
from .installers.codex_installer import CodexInstaller
from .installers.conda_installer import CondaInstaller
from .installers.node_installer import NodeInstaller
from .installers.nvm_installer import NVMInstaller
from .installers.spec_kit_installer import SpecKitInstaller
from .installers.uv_installer import UVInstaller
from .mirror_config import MirrorConfigurator
from .platform_detector import PlatformInfo
from .project_creator import ProjectCreator
from .tool_detector import ToolDetector


# 工具安装顺序（基于依赖关系）
INSTALL_ORDER = [
    "nvm",           # 1. 首先安装 NVM
    "node",          # 2. 通过 NVM 安装 Node.js
    "conda",         # 3. 安装 Conda（独立）
    "bun",           # 4. 安装 Bun（需要 Node.js 作为备选）
    "uv",            # 5. 安装 uv
    "claude-code",   # 6. 安装 Claude Code CLI
    "codex",         # 7. 安装 Codex CLI（可能依赖 Bun）
    "spec-kit",      # 8. 安装 Spec Kit（依赖 uv）
    "bmad-method",   # 9. 安装 BMad Method（依赖 Node.js/Bun）
]


class InstallOrchestrator:
    """安装编排器
    
    负责编排整个安装和升级流程。
    
    Attributes:
        config: 配置对象
        platform_info: 平台信息
        dry_run: 是否为模拟运行
        tool_detector: 工具检测器
        mirror_configurator: 镜像源配置器
    """
    
    def __init__(
        self,
        config: Config,
        platform_info: PlatformInfo,
        dry_run: bool = False
    ):
        """初始化安装编排器
        
        Args:
            config: 配置对象
            platform_info: 平台信息
            dry_run: 是否为模拟运行（默认 False）
        """
        self.config = config
        self.platform_info = platform_info
        self.dry_run = dry_run
        self.tool_detector = ToolDetector()
        self.mirror_configurator = MirrorConfigurator(config.registry)
    
    def get_install_order(self) -> List[str]:
        """获取工具安装顺序
        
        根据配置中启用的工具，返回按依赖关系排序的工具列表。
        
        Returns:
            List[str]: 工具名称列表，按安装顺序排列
        """
        # 获取所有启用的工具
        enabled_tools = []
        for tool_name in INSTALL_ORDER:
            tool_config = self.config.tools.get(tool_name, ToolConfig())
            if tool_config.enabled:
                enabled_tools.append(tool_name)
        
        return enabled_tools
    
    def _create_installer(self, tool_name: str) -> Optional[ToolInstaller]:
        """创建工具安装器实例
        
        Args:
            tool_name: 工具名称
            
        Returns:
            Optional[ToolInstaller]: 安装器实例，如果工具名称无效则返回 None
        """
        tool_config = self.config.tools.get(tool_name, ToolConfig())
        
        installer_map = {
            "nvm": NVMInstaller,
            "node": NodeInstaller,
            "conda": CondaInstaller,
            "bun": BunInstaller,
            "uv": UVInstaller,
            "claude-code": ClaudeCodeInstaller,
            "codex": CodexInstaller,
            "spec-kit": SpecKitInstaller,
            "bmad-method": BMadInstaller,
        }
        
        installer_class = installer_map.get(tool_name)
        if installer_class is None:
            return None
        
        return installer_class(self.platform_info, tool_config)
    
    def install_tool(self, tool_name: str) -> InstallReport:
        """安装单个工具
        
        执行以下步骤：
        1. 检测工具是否已安装
        2. 如果已安装，跳过安装
        3. 如果未安装，创建安装器并执行安装
        
        Args:
            tool_name: 工具名称
            
        Returns:
            InstallReport: 安装报告
        """
        # 如果是模拟运行，返回模拟报告
        if self.dry_run:
            return InstallReport(
                tool_name=tool_name,
                result=InstallResult.SKIPPED,
                message=f"[模拟运行] 将安装 {tool_name}"
            )
        
        # 创建安装器
        installer = self._create_installer(tool_name)
        if installer is None:
            return InstallReport(
                tool_name=tool_name,
                result=InstallResult.FAILED,
                message=f"无效的工具名称: {tool_name}",
                error="工具名称不在支持列表中"
            )
        
        # 执行安装
        try:
            return installer.install()
        except Exception as e:
            return InstallReport(
                tool_name=tool_name,
                result=InstallResult.FAILED,
                message=f"{tool_name} 安装过程中发生异常",
                error=str(e)
            )
    
    def install_all_tools(self) -> Dict[str, InstallReport]:
        """按顺序安装所有工具
        
        按照依赖关系顺序安装所有启用的工具。
        即使某个工具安装失败，也会继续安装其余工具（容错性）。
        
        Returns:
            Dict[str, InstallReport]: 工具名称到安装报告的映射
        """
        install_order = self.get_install_order()
        reports = {}
        
        for tool_name in install_order:
            report = self.install_tool(tool_name)
            reports[tool_name] = report
        
        return reports
    
    def configure_mirrors(self) -> Dict[str, bool]:
        """配置镜像源
        
        配置 npm、Bun 和 uv 的镜像源。
        
        Returns:
            Dict[str, bool]: 工具名称到配置结果的映射
        """
        if self.dry_run:
            return {
                "npm": True,
                "bun": True,
                "uv": True,
            }
        
        return self.mirror_configurator.configure_all()
    
    def create_project(self, project_name: Optional[str] = None, force: bool = False) -> tuple[bool, Optional[str]]:
        """创建项目结构
        
        Args:
            project_name: 项目名称（如果为 None，使用配置中的名称或当前目录名）
            force: 是否强制覆盖已存在的目录
            
        Returns:
            tuple[bool, Optional[str]]: (是否成功, 错误信息)
        """
        if self.dry_run:
            return True, None
        
        # 确定项目名称
        if project_name is None:
            project_name = self.config.project.name
        
        if project_name is None:
            project_name = Path.cwd().name
        
        # 创建项目
        project_path = Path.cwd() / project_name
        creator = ProjectCreator(project_name, project_path)
        
        return creator.create_project(force=force)
    
    def run_init(self, project_name: Optional[str] = None, force: bool = False) -> Dict[str, InstallReport]:
        """执行完整初始化流程
        
        执行以下步骤：
        1. 安装所有工具
        2. 配置镜像源
        3. 创建项目结构
        
        Args:
            project_name: 项目名称
            force: 是否强制覆盖已存在的目录
            
        Returns:
            Dict[str, InstallReport]: 所有工具的安装报告
        """
        # 1. 安装所有工具
        tool_reports = self.install_all_tools()
        
        # 2. 配置镜像源（在相关工具安装成功后）
        # 检查 npm/node 是否安装成功
        if "node" in tool_reports and tool_reports["node"].result == InstallResult.SUCCESS:
            npm_result = self.mirror_configurator.configure_npm_mirror()
            tool_reports["npm-mirror"] = InstallReport(
                tool_name="npm-mirror",
                result=InstallResult.SUCCESS if npm_result else InstallResult.FAILED,
                message="npm 镜像源配置成功" if npm_result else "npm 镜像源配置失败"
            )
        
        # 检查 bun 是否安装成功
        if "bun" in tool_reports and tool_reports["bun"].result == InstallResult.SUCCESS:
            bun_result = self.mirror_configurator.configure_bun_mirror()
            tool_reports["bun-mirror"] = InstallReport(
                tool_name="bun-mirror",
                result=InstallResult.SUCCESS if bun_result else InstallResult.FAILED,
                message="Bun 镜像源配置成功" if bun_result else "Bun 镜像源配置失败"
            )
        
        # 检查 uv 是否安装成功
        if "uv" in tool_reports and tool_reports["uv"].result == InstallResult.SUCCESS:
            uv_result = self.mirror_configurator.configure_uv_mirror()
            tool_reports["uv-mirror"] = InstallReport(
                tool_name="uv-mirror",
                result=InstallResult.SUCCESS if uv_result else InstallResult.FAILED,
                message="uv 镜像源配置成功" if uv_result else "uv 镜像源配置失败"
            )
        
        # 3. 创建项目结构
        if not self.dry_run:
            project_success, project_error = self.create_project(project_name, force)
            tool_reports["project"] = InstallReport(
                tool_name="project",
                result=InstallResult.SUCCESS if project_success else InstallResult.FAILED,
                message="项目创建成功" if project_success else "项目创建失败",
                error=project_error
            )
        
        return tool_reports
    
    def upgrade_tool(self, tool_name: str) -> InstallReport:
        """升级单个工具
        
        Args:
            tool_name: 工具名称
            
        Returns:
            InstallReport: 升级报告
        """
        # 如果是模拟运行，返回模拟报告
        if self.dry_run:
            return InstallReport(
                tool_name=tool_name,
                result=InstallResult.SKIPPED,
                message=f"[模拟运行] 将升级 {tool_name}"
            )
        
        # 创建安装器
        installer = self._create_installer(tool_name)
        if installer is None:
            return InstallReport(
                tool_name=tool_name,
                result=InstallResult.FAILED,
                message=f"无效的工具名称: {tool_name}",
                error="工具名称不在支持列表中"
            )
        
        # 执行升级
        try:
            return installer.upgrade()
        except Exception as e:
            return InstallReport(
                tool_name=tool_name,
                result=InstallResult.FAILED,
                message=f"{tool_name} 升级过程中发生异常",
                error=str(e)
            )
    
    def run_upgrade(self, tool_name: Optional[str] = None) -> Dict[str, InstallReport]:
        """执行升级流程
        
        Args:
            tool_name: 要升级的工具名称（如果为 None，升级所有工具）
            
        Returns:
            Dict[str, InstallReport]: 工具名称到升级报告的映射
        """
        reports = {}
        
        if tool_name is not None:
            # 升级单个工具
            report = self.upgrade_tool(tool_name)
            reports[tool_name] = report
        else:
            # 升级所有已安装的工具
            all_tools = self.tool_detector.detect_all_tools()
            
            for tool_name, tool_status in all_tools.items():
                if tool_status.installed:
                    report = self.upgrade_tool(tool_name)
                    reports[tool_name] = report
        
        return reports
    
    def print_summary(self, reports: Dict[str, InstallReport]) -> None:
        """打印安装摘要
        
        显示成功、跳过和失败的工具列表。
        
        Args:
            reports: 工具名称到安装报告的映射
        """
        # 分类统计
        success_tools = []
        skipped_tools = []
        failed_tools = []
        
        for tool_name, report in reports.items():
            if report.result == InstallResult.SUCCESS:
                success_tools.append((tool_name, report))
            elif report.result == InstallResult.SKIPPED:
                skipped_tools.append((tool_name, report))
            elif report.result == InstallResult.FAILED:
                failed_tools.append((tool_name, report))
        
        # 打印摘要
        print("\n" + "=" * 60)
        print("安装摘要")
        print("=" * 60)
        
        # 成功的工具
        if success_tools:
            print(f"\n✓ 成功 ({len(success_tools)}):")
            for tool_name, report in success_tools:
                version_info = f" (v{report.version})" if report.version else ""
                print(f"  - {tool_name}{version_info}: {report.message}")
        
        # 跳过的工具
        if skipped_tools:
            print(f"\n○ 跳过 ({len(skipped_tools)}):")
            for tool_name, report in skipped_tools:
                version_info = f" (v{report.version})" if report.version else ""
                print(f"  - {tool_name}{version_info}: {report.message}")
        
        # 失败的工具
        if failed_tools:
            print(f"\n✗ 失败 ({len(failed_tools)}):")
            for tool_name, report in failed_tools:
                print(f"  - {tool_name}: {report.message}")
                if report.error:
                    print(f"    错误: {report.error}")
        
        print("\n" + "=" * 60)
        
        # 总结
        total = len(reports)
        print(f"总计: {total} 个任务")
        print(f"  成功: {len(success_tools)}")
        print(f"  跳过: {len(skipped_tools)}")
        print(f"  失败: {len(failed_tools)}")
        print("=" * 60 + "\n")



"""单元测试：BMad Method 安装器

该模块测试 BMad Method 安装器的各种场景。
"""

import pytest
from unittest.mock import Mock, patch, MagicMock
from pathlib import Path

from src.mono_kickstart.config import ToolConfig
from src.mono_kickstart.installer_base import InstallResult
from src.mono_kickstart.installers.bmad_installer import BMadInstaller
from src.mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell


@pytest.fixture
def platform_info():
    """创建测试用的平台信息"""
    return PlatformInfo(
        os=OS.LINUX,
        arch=Arch.X86_64,
        shell=Shell.BASH,
        shell_config_file="/home/test/.bashrc"
    )


@pytest.fixture
def tool_config():
    """创建测试用的工具配置"""
    return ToolConfig(enabled=True)


@pytest.fixture
def installer(platform_info, tool_config):
    """创建 BMad Method 安装器实例"""
    return BMadInstaller(platform_info, tool_config)


class TestBMadInstallerDetermineInstallMethod:
    """测试 BMad Method 安装器的安装方式确定功能"""
    
    def test_determine_install_method_with_config(self, platform_info):
        """测试配置中指定安装方式时使用配置的方式"""
        config = ToolConfig(enabled=True, install_via="npx")
        installer = BMadInstaller(platform_info, config)
        assert installer.install_method == "npx"
    
    def test_determine_install_method_with_bun_available(self, platform_info, tool_config):
        """测试 Bun 可用时优先使用 bunx"""
        with patch('shutil.which', side_effect=lambda cmd: "/usr/local/bin/bun" if cmd == "bun" else None):
            installer = BMadInstaller(platform_info, tool_config)
            assert installer.install_method == "bunx"
    
    def test_determine_install_method_without_bun(self, platform_info, tool_config):
        """测试 Bun 不可用时使用 npx"""
        with patch('shutil.which', return_value=None):
            installer = BMadInstaller(platform_info, tool_config)
            assert installer.install_method == "npx"


class TestBMadInstallerVerify:
    """测试 BMad Method 安装器的验证功能"""
    
    def test_verify_when_bmad_installed(self, installer):
        """测试 BMad Method 已安装时的验证"""
        with patch('shutil.which', return_value="/usr/local/bin/bmad"), \
             patch.object(installer, 'run_command', return_value=(0, "bmad 1.0.0", "")):
            assert installer.verify() is True
    
    def test_verify_when_bmad_not_in_path(self, installer):
        """测试 BMad Method 不在 PATH 中时的验证"""
        with patch('shutil.which', return_value=None):
            assert installer.verify() is False
    
    def test_verify_when_bmad_command_fails(self, installer):
        """测试 BMad Method 命令执行失败时的验证"""
        with patch('shutil.which', return_value="/usr/local/bin/bmad"), \
             patch.object(installer, 'run_command', return_value=(1, "", "error")):
            assert installer.verify() is False


class TestBMadInstallerGetVersion:
    """测试 BMad Method 安装器的版本获取功能"""
    
    def test_get_version_success(self, installer):
        """测试成功获取版本"""
        with patch('shutil.which', return_value="/usr/local/bin/bmad"), \
             patch.object(installer, 'run_command', return_value=(0, "1.2.3", "")):
            version = installer._get_installed_version()
            assert version == "1.2.3"
    
    def test_get_version_when_not_installed(self, installer):
        """测试 BMad Method 未安装时获取版本"""
        with patch('shutil.which', return_value=None):
            version = installer._get_installed_version()
            assert version is None
    
    def test_get_version_when_command_fails(self, installer):
        """测试命令失败时获取版本"""
        with patch('shutil.which', return_value="/usr/local/bin/bmad"), \
             patch.object(installer, 'run_command', return_value=(1, "", "error")):
            version = installer._get_installed_version()
            assert version is None


class TestBMadInstallerInstall:
    """测试 BMad Method 安装器的安装功能"""
    
    def test_install_when_already_installed(self, installer):
        """测试 BMad Method 已安装时跳过安装"""
        with patch.object(installer, 'verify', return_value=True), \
             patch.object(installer, '_get_installed_version', return_value="1.0.0"):
            report = installer.install()
            
            assert report.result == InstallResult.SKIPPED
            assert report.tool_name == "bmad-method"
            assert report.version == "1.0.0"
            assert "已安装" in report.message
    
    def test_install_with_bunx_when_bun_not_available(self, platform_info):
        """测试使用 bunx 但 Bun 未安装时失败"""
        config = ToolConfig(enabled=True, install_via="bunx")
        installer = BMadInstaller(platform_info, config)
        
        with patch.object(installer, 'verify', return_value=False), \
             patch('shutil.which', return_value=None):
            report = installer.install()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "bmad-method"
            assert "Bun 未安装" in report.message
    
    def test_install_with_npx_when_npx_not_available(self, platform_info):
        """测试使用 npx 但 npx 未安装时失败"""
        config = ToolConfig(enabled=True, install_via="npx")
        installer = BMadInstaller(platform_info, config)
        
        with patch.object(installer, 'verify', return_value=False), \
             patch('shutil.which', return_value=None):
            report = installer.install()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "bmad-method"
            assert "npx 未安装" in report.message
    
    def test_install_success_with_bunx(self, platform_info):
        """测试使用 bunx 成功安装 BMad Method"""
        config = ToolConfig(enabled=True, install_via="bunx")
        installer = BMadInstaller(platform_info, config)
        
        with patch.object(installer, 'verify', side_effect=[False, True]), \
             patch('shutil.which', return_value="/usr/local/bin/bun"), \
             patch.object(installer, 'run_command', return_value=(0, "installed", "")), \
             patch.object(installer, '_get_installed_version', return_value="1.2.3"):
            report = installer.install()
            
            assert report.result == InstallResult.SUCCESS
            assert report.tool_name == "bmad-method"
            assert report.version == "1.2.3"
            assert "成功" in report.message
            assert "bunx" in report.message
    
    def test_install_success_with_npx(self, platform_info):
        """测试使用 npx 成功安装 BMad Method"""
        config = ToolConfig(enabled=True, install_via="npx")
        installer = BMadInstaller(platform_info, config)
        
        with patch.object(installer, 'verify', side_effect=[False, True]), \
             patch('shutil.which', return_value="/usr/local/bin/npx"), \
             patch.object(installer, 'run_command', return_value=(0, "installed", "")), \
             patch.object(installer, '_get_installed_version', return_value="1.2.3"):
            report = installer.install()
            
            assert report.result == InstallResult.SUCCESS
            assert report.tool_name == "bmad-method"
            assert report.version == "1.2.3"
            assert "成功" in report.message
            assert "npx" in report.message
    
    def test_install_command_fails(self, installer):
        """测试安装命令执行失败"""
        with patch.object(installer, 'verify', return_value=False), \
             patch('shutil.which', return_value="/usr/local/bin/npx"), \
             patch.object(installer, 'run_command', return_value=(1, "", "install failed")):
            report = installer.install()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "bmad-method"
            assert "失败" in report.message
            assert report.error is not None
    
    def test_install_verification_fails(self, installer):
        """测试安装后验证失败"""
        with patch.object(installer, 'verify', side_effect=[False, False]), \
             patch('shutil.which', return_value="/usr/local/bin/npx"), \
             patch.object(installer, 'run_command', return_value=(0, "installed", "")):
            report = installer.install()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "bmad-method"
            assert "验证失败" in report.message
    
    def test_install_exception(self, installer):
        """测试安装过程中发生异常"""
        with patch.object(installer, 'verify', return_value=False), \
             patch('shutil.which', return_value="/usr/local/bin/npx"), \
             patch.object(installer, 'run_command', side_effect=Exception("test error")):
            report = installer.install()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "bmad-method"
            assert "异常" in report.message
            assert "test error" in report.error


class TestBMadInstallerUpgrade:
    """测试 BMad Method 安装器的升级功能"""
    
    def test_upgrade_when_not_installed(self, installer):
        """测试 BMad Method 未安装时无法升级"""
        with patch.object(installer, 'verify', return_value=False):
            report = installer.upgrade()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "bmad-method"
            assert "未安装" in report.message
    
    def test_upgrade_with_bunx_when_bun_not_available(self, platform_info):
        """测试使用 bunx 升级但 Bun 未安装时失败"""
        config = ToolConfig(enabled=True, install_via="bunx")
        installer = BMadInstaller(platform_info, config)
        
        with patch.object(installer, 'verify', return_value=True), \
             patch('shutil.which', return_value=None):
            report = installer.upgrade()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "bmad-method"
            assert "Bun 未安装" in report.message
    
    def test_upgrade_with_npx_when_npx_not_available(self, platform_info):
        """测试使用 npx 升级但 npx 未安装时失败"""
        config = ToolConfig(enabled=True, install_via="npx")
        installer = BMadInstaller(platform_info, config)
        
        with patch.object(installer, 'verify', return_value=True), \
             patch('shutil.which', return_value=None):
            report = installer.upgrade()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "bmad-method"
            assert "npx 未安装" in report.message
    
    def test_upgrade_success_with_bunx(self, platform_info):
        """测试使用 bunx 成功升级 BMad Method"""
        config = ToolConfig(enabled=True, install_via="bunx")
        installer = BMadInstaller(platform_info, config)
        
        with patch.object(installer, 'verify', return_value=True), \
             patch('shutil.which', return_value="/usr/local/bin/bun"), \
             patch.object(installer, '_get_installed_version', side_effect=["1.0.0", "1.2.3"]), \
             patch.object(installer, 'run_command', return_value=(0, "upgraded", "")):
            report = installer.upgrade()
            
            assert report.result == InstallResult.SUCCESS
            assert report.tool_name == "bmad-method"
            assert report.version == "1.2.3"
            assert "1.0.0" in report.message
            assert "1.2.3" in report.message
    
    def test_upgrade_success_with_npx(self, platform_info):
        """测试使用 npx 成功升级 BMad Method"""
        config = ToolConfig(enabled=True, install_via="npx")
        installer = BMadInstaller(platform_info, config)
        
        with patch.object(installer, 'verify', return_value=True), \
             patch('shutil.which', return_value="/usr/local/bin/npx"), \
             patch.object(installer, '_get_installed_version', side_effect=["1.0.0", "1.2.3"]), \
             patch.object(installer, 'run_command', return_value=(0, "upgraded", "")):
            report = installer.upgrade()
            
            assert report.result == InstallResult.SUCCESS
            assert report.tool_name == "bmad-method"
            assert report.version == "1.2.3"
            assert "1.0.0" in report.message
            assert "1.2.3" in report.message
    
    def test_upgrade_command_fails(self, installer):
        """测试升级命令执行失败"""
        with patch.object(installer, 'verify', return_value=True), \
             patch('shutil.which', return_value="/usr/local/bin/npx"), \
             patch.object(installer, '_get_installed_version', return_value="1.0.0"), \
             patch.object(installer, 'run_command', return_value=(1, "", "upgrade failed")):
            report = installer.upgrade()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "bmad-method"
            assert "失败" in report.message
    
    def test_upgrade_verification_fails(self, installer):
        """测试升级后验证失败"""
        with patch.object(installer, 'verify', side_effect=[True, False]), \
             patch('shutil.which', return_value="/usr/local/bin/npx"), \
             patch.object(installer, '_get_installed_version', return_value="1.0.0"), \
             patch.object(installer, 'run_command', return_value=(0, "upgraded", "")):
            report = installer.upgrade()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "bmad-method"
            assert "验证失败" in report.message
    
    def test_upgrade_exception(self, installer):
        """测试升级过程中发生异常"""
        with patch.object(installer, 'verify', return_value=True), \
             patch('shutil.which', return_value="/usr/local/bin/npx"), \
             patch.object(installer, '_get_installed_version', side_effect=Exception("test error")):
            report = installer.upgrade()
            
            assert report.result == InstallResult.FAILED
            assert report.tool_name == "bmad-method"
            assert "异常" in report.message
            assert "test error" in report.error


class TestBMadInstallerCommands:
    """测试 BMad Method 安装器使用正确的命令"""
    
    def test_install_uses_bunx_command(self, platform_info):
        """测试安装时使用正确的 bunx 命令"""
        config = ToolConfig(enabled=True, install_via="bunx")
        installer = BMadInstaller(platform_info, config)
        
        with patch.object(installer, 'verify', side_effect=[False, True]), \
             patch('shutil.which', return_value="/usr/local/bin/bun"), \
             patch.object(installer, '_get_installed_version', return_value="1.0.0"):
            
            # Mock run_command to capture the command
            commands_run = []
            def capture_command(cmd, **kwargs):
                commands_run.append(cmd)
                return (0, "installed", "")
            
            with patch.object(installer, 'run_command', side_effect=capture_command):
                report = installer.install()
            
            # Verify the command is correct
            assert len(commands_run) == 1
            assert "bunx bmad-method init" in commands_run[0]
    
    def test_install_uses_npx_command(self, platform_info):
        """测试安装时使用正确的 npx 命令"""
        config = ToolConfig(enabled=True, install_via="npx")
        installer = BMadInstaller(platform_info, config)
        
        with patch.object(installer, 'verify', side_effect=[False, True]), \
             patch('shutil.which', return_value="/usr/local/bin/npx"), \
             patch.object(installer, '_get_installed_version', return_value="1.0.0"):
            
            # Mock run_command to capture the command
            commands_run = []
            def capture_command(cmd, **kwargs):
                commands_run.append(cmd)
                return (0, "installed", "")
            
            with patch.object(installer, 'run_command', side_effect=capture_command):
                report = installer.install()
            
            # Verify the command is correct
            assert len(commands_run) == 1
            assert "npx bmad-method init" in commands_run[0]
    
    def test_upgrade_uses_bunx_latest_command(self, platform_info):
        """测试升级时使用正确的 bunx @latest 命令"""
        config = ToolConfig(enabled=True, install_via="bunx")
        installer = BMadInstaller(platform_info, config)
        
        with patch.object(installer, 'verify', return_value=True), \
             patch('shutil.which', return_value="/usr/local/bin/bun"), \
             patch.object(installer, '_get_installed_version', side_effect=["1.0.0", "1.2.3"]):
            
            # Mock run_command to capture the command
            commands_run = []
            def capture_command(cmd, **kwargs):
                commands_run.append(cmd)
                return (0, "upgraded", "")
            
            with patch.object(installer, 'run_command', side_effect=capture_command):
                report = installer.upgrade()
            
            # Verify the command includes @latest
            assert len(commands_run) == 1
            assert "bunx bmad-method@latest init" in commands_run[0]
    
    def test_upgrade_uses_npx_latest_command(self, platform_info):
        """测试升级时使用正确的 npx @latest 命令"""
        config = ToolConfig(enabled=True, install_via="npx")
        installer = BMadInstaller(platform_info, config)
        
        with patch.object(installer, 'verify', return_value=True), \
             patch('shutil.which', return_value="/usr/local/bin/npx"), \
             patch.object(installer, '_get_installed_version', side_effect=["1.0.0", "1.2.3"]):
            
            # Mock run_command to capture the command
            commands_run = []
            def capture_command(cmd, **kwargs):
                commands_run.append(cmd)
                return (0, "upgraded", "")
            
            with patch.object(installer, 'run_command', side_effect=capture_command):
                report = installer.upgrade()
            
            # Verify the command includes @latest
            assert len(commands_run) == 1
            assert "npx bmad-method@latest init" in commands_run[0]



"""属性测试：Codex CLI 安装方式选择

**Validates: Requirements 1.9**

该模块测试 Codex CLI 安装器的条件安装方式选择逻辑。
"""

import pytest
from hypothesis import given, strategies as st
from unittest.mock import Mock, patch

from src.mono_kickstart.config import ToolConfig
from src.mono_kickstart.installers.codex_installer import CodexInstaller
from src.mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell


# 策略：生成平台信息
@st.composite
def platform_info_strategy(draw):
    """生成平台信息策略"""
    os = draw(st.sampled_from([OS.MACOS, OS.LINUX]))
    arch = draw(st.sampled_from([Arch.ARM64, Arch.X86_64]))
    shell = draw(st.sampled_from([Shell.BASH, Shell.ZSH, Shell.FISH]))
    shell_config_file = f"/home/user/.{shell.value}rc"
    
    return PlatformInfo(
        os=os,
        arch=arch,
        shell=shell,
        shell_config_file=shell_config_file
    )


# 策略：生成工具配置
@st.composite
def tool_config_strategy(draw):
    """生成工具配置策略"""
    enabled = draw(st.booleans())
    version = draw(st.one_of(st.none(), st.text(min_size=1, max_size=20)))
    install_via = draw(st.one_of(st.none(), st.sampled_from(['bun', 'npm', 'Bun', 'NPM'])))
    
    return ToolConfig(
        enabled=enabled,
        version=version,
        install_via=install_via
    )


# Feature: mono-kickstart, Property 4: 条件安装方式选择
@given(
    platform_info=platform_info_strategy(),
    config=tool_config_strategy(),
    bun_installed=st.booleans()
)
def test_codex_install_method_selection(platform_info, config, bun_installed):
    """
    属性 4: 条件安装方式选择
    
    对于任何工具安装状态，当安装 Codex CLI 时，如果 Bun 已安装则应使用 bun install -g，
    否则应使用 npm install -g。
    
    **Validates: Requirements 1.9**
    """
    with patch('shutil.which') as mock_which:
        # 模拟 Bun 的安装状态
        def which_side_effect(cmd):
            if cmd == "bun":
                return "/usr/local/bin/bun" if bun_installed else None
            return None
        
        mock_which.side_effect = which_side_effect
        
        # 创建安装器
        installer = CodexInstaller(platform_info, config)
        
        # 验证安装方式选择逻辑
        if config.install_via:
            # 如果配置中指定了安装方式，应使用指定的方式
            expected_method = config.install_via.lower()
            assert installer.install_method == expected_method, \
                f"配置指定了 install_via={config.install_via}，应使用 {expected_method}"
        else:
            # 如果配置中未指定，应根据 Bun 是否安装来选择
            if bun_installed:
                assert installer.install_method == 'bun', \
                    "Bun 已安装，应使用 bun 作为安装方式"
            else:
                assert installer.install_method == 'npm', \
                    "Bun 未安装，应使用 npm 作为安装方式"


# Feature: mono-kickstart, Property 4: 条件安装方式选择（安装命令验证）
@given(
    platform_info=platform_info_strategy(),
    bun_installed=st.booleans()
)
def test_codex_install_command_matches_method(platform_info, bun_installed):
    """
    属性 4: 条件安装方式选择（安装命令验证）
    
    验证实际执行的安装命令与选择的安装方式一致。
    
    **Validates: Requirements 1.9**
    """
    with patch('shutil.which') as mock_which, \
         patch.object(CodexInstaller, 'run_command') as mock_run_command, \
         patch.object(CodexInstaller, 'verify') as mock_verify:
        
        # 模拟 Bun 的安装状态
        def which_side_effect(cmd):
            if cmd == "bun":
                return "/usr/local/bin/bun" if bun_installed else None
            elif cmd == "npm":
                return "/usr/local/bin/npm"
            elif cmd == "codex":
                return None  # 未安装
            return None
        
        mock_which.side_effect = which_side_effect
        
        # 模拟命令执行成功
        mock_run_command.return_value = (0, "success", "")
        
        # 模拟验证：第一次返回 False（未安装），第二次返回 True（安装成功）
        mock_verify.side_effect = [False, True]
        
        # 创建安装器（不指定 install_via）
        config = ToolConfig(enabled=True)
        installer = CodexInstaller(platform_info, config)
        
        # 执行安装
        result = installer.install()
        
        # 验证安装命令
        if bun_installed:
            # 应该调用 bun install -g
            assert mock_run_command.called
            call_args = mock_run_command.call_args[0][0]
            assert "bun install -g @openai/codex" in call_args, \
                f"Bun 已安装，应使用 'bun install -g' 命令，实际命令: {call_args}"
        else:
            # 应该调用 npm install -g
            assert mock_run_command.called
            call_args = mock_run_command.call_args[0][0]
            assert "npm i -g @openai/codex" in call_args, \
                f"Bun 未安装，应使用 'npm install -g' 命令，实际命令: {call_args}"


# Feature: mono-kickstart, Property 4: 条件安装方式选择（升级命令验证）
@given(
    platform_info=platform_info_strategy(),
    bun_installed=st.booleans()
)
def test_codex_upgrade_command_matches_method(platform_info, bun_installed):
    """
    属性 4: 条件安装方式选择（升级命令验证）
    
    验证实际执行的升级命令与选择的安装方式一致。
    
    **Validates: Requirements 1.9**
    """
    with patch('shutil.which') as mock_which, \
         patch.object(CodexInstaller, 'run_command') as mock_run_command, \
         patch.object(CodexInstaller, 'verify') as mock_verify, \
         patch.object(CodexInstaller, '_get_installed_version') as mock_get_version:
        
        # 模拟 Bun 的安装状态
        def which_side_effect(cmd):
            if cmd == "bun":
                return "/usr/local/bin/bun" if bun_installed else None
            elif cmd == "npm":
                return "/usr/local/bin/npm"
            elif cmd == "codex":
                return "/usr/local/bin/codex"  # 已安装
            return None
        
        mock_which.side_effect = which_side_effect
        
        # 模拟命令执行成功
        mock_run_command.return_value = (0, "success", "")
        
        # 模拟验证成功
        mock_verify.return_value = True
        
        # 模拟版本获取
        mock_get_version.return_value = "1.0.0"
        
        # 创建安装器（不指定 install_via）
        config = ToolConfig(enabled=True)
        installer = CodexInstaller(platform_info, config)
        
        # 执行升级
        result = installer.upgrade()
        
        # 验证升级命令
        if bun_installed:
            # 应该调用 bun update -g
            assert mock_run_command.called
            call_args = mock_run_command.call_args[0][0]
            assert "bun update -g @openai/codex" in call_args, \
                f"Bun 已安装，应使用 'bun update -g' 命令，实际命令: {call_args}"
        else:
            # 应该调用 npm update -g
            assert mock_run_command.called
            call_args = mock_run_command.call_args[0][0]
            assert "npm update -g @openai/codex" in call_args, \
                f"Bun 未安装，应使用 'npm update -g' 命令，实际命令: {call_args}"



"""
工具检测器模块

检测开发工具是否已安装及其版本信息。
"""

import os
import re
import shutil
import subprocess
from dataclasses import dataclass
from pathlib import Path
from typing import Dict, Optional


@dataclass
class ToolStatus:
    """工具状态
    
    Attributes:
        name: 工具名称
        installed: 是否已安装
        version: 版本号（如果已安装）
        path: 可执行文件路径（如果已安装）
    """
    name: str
    installed: bool
    version: Optional[str] = None
    path: Optional[str] = None


class ToolDetector:
    """工具检测器
    
    检测各种开发工具的安装状态和版本信息。
    """
    
    def is_command_available(self, command: str) -> bool:
        """检查命令是否在 PATH 中可用
        
        Args:
            command: 命令名称
            
        Returns:
            bool: 如果命令可用返回 True，否则返回 False
        """
        return shutil.which(command) is not None
    
    def get_command_version(
        self,
        command: str,
        version_arg: str = "--version"
    ) -> Optional[str]:
        """获取命令的版本号
        
        Args:
            command: 命令名称
            version_arg: 获取版本的参数（默认为 --version）
            
        Returns:
            Optional[str]: 版本号字符串，如果无法获取则返回 None
        """
        try:
            result = subprocess.run(
                [command, version_arg],
                capture_output=True,
                text=True,
                timeout=5
            )
            
            # 尝试从 stdout 或 stderr 中提取版本号
            output = result.stdout + result.stderr
            
            # 常见的版本号模式
            # 匹配 v1.2.3, 1.2.3, version 1.2.3 等格式
            version_patterns = [
                r'v?(\d+\.\d+\.\d+(?:\.\d+)?)',  # 1.2.3 或 v1.2.3
                r'version\s+v?(\d+\.\d+\.\d+)',   # version 1.2.3
                r'(\d+\.\d+\.\d+)',                # 简单的 1.2.3
            ]
            
            for pattern in version_patterns:
                match = re.search(pattern, output, re.IGNORECASE)
                if match:
                    return match.group(1)
            
            # 如果没有匹配到标准版本号，返回第一行（去除空白）
            first_line = output.strip().split('\n')[0] if output.strip() else None
            return first_line
            
        except (subprocess.TimeoutExpired, subprocess.SubprocessError, FileNotFoundError):
            return None
    
    def detect_nvm(self) -> ToolStatus:
        """检测 NVM
        
        NVM 是一个 shell 函数，不是独立的可执行文件，需要特殊处理。
        
        Returns:
            ToolStatus: NVM 的状态信息
        """
        # NVM 通常安装在 ~/.nvm 目录
        nvm_dir = Path.home() / ".nvm"
        nvm_sh = nvm_dir / "nvm.sh"
        
        if nvm_sh.exists():
            # 尝试通过 source nvm.sh 并执行 nvm --version 获取版本
            try:
                # 使用 bash -c 来 source nvm.sh 并获取版本
                result = subprocess.run(
                    ["bash", "-c", f"source {nvm_sh} && nvm --version"],
                    capture_output=True,
                    text=True,
                    timeout=5
                )
                
                if result.returncode == 0:
                    version = result.stdout.strip()
                    return ToolStatus(
                        name="nvm",
                        installed=True,
                        version=version,
                        path=str(nvm_sh)
                    )
            except (subprocess.TimeoutExpired, subprocess.SubprocessError):
                pass
            
            # 如果无法获取版本，但文件存在，仍然认为已安装
            return ToolStatus(
                name="nvm",
                installed=True,
                version=None,
                path=str(nvm_sh)
            )
        
        return ToolStatus(name="nvm", installed=False)
    
    def detect_node(self) -> ToolStatus:
        """检测 Node.js
        
        Returns:
            ToolStatus: Node.js 的状态信息
        """
        if not self.is_command_available("node"):
            return ToolStatus(name="node", installed=False)
        
        version = self.get_command_version("node", "--version")
        path = shutil.which("node")
        
        return ToolStatus(
            name="node",
            installed=True,
            version=version,
            path=path
        )
    
    def detect_conda(self) -> ToolStatus:
        """检测 Conda
        
        Returns:
            ToolStatus: Conda 的状态信息
        """
        if not self.is_command_available("conda"):
            return ToolStatus(name="conda", installed=False)
        
        version = self.get_command_version("conda", "--version")
        path = shutil.which("conda")
        
        return ToolStatus(
            name="conda",
            installed=True,
            version=version,
            path=path
        )
    
    def detect_bun(self) -> ToolStatus:
        """检测 Bun
        
        Returns:
            ToolStatus: Bun 的状态信息
        """
        if not self.is_command_available("bun"):
            return ToolStatus(name="bun", installed=False)
        
        version = self.get_command_version("bun", "--version")
        path = shutil.which("bun")
        
        return ToolStatus(
            name="bun",
            installed=True,
            version=version,
            path=path
        )
    
    def detect_uv(self) -> ToolStatus:
        """检测 uv
        
        Returns:
            ToolStatus: uv 的状态信息
        """
        if not self.is_command_available("uv"):
            return ToolStatus(name="uv", installed=False)
        
        version = self.get_command_version("uv", "--version")
        path = shutil.which("uv")
        
        return ToolStatus(
            name="uv",
            installed=True,
            version=version,
            path=path
        )
    
    def detect_claude_code(self) -> ToolStatus:
        """检测 Claude Code CLI
        
        Returns:
            ToolStatus: Claude Code CLI 的状态信息
        """
        if not self.is_command_available("claude"):
            return ToolStatus(name="claude-code", installed=False)
        
        version = self.get_command_version("claude", "--version")
        path = shutil.which("claude")
        
        return ToolStatus(
            name="claude-code",
            installed=True,
            version=version,
            path=path
        )
    
    def detect_codex(self) -> ToolStatus:
        """检测 Codex CLI
        
        Returns:
            ToolStatus: Codex CLI 的状态信息
        """
        if not self.is_command_available("codex"):
            return ToolStatus(name="codex", installed=False)
        
        version = self.get_command_version("codex", "--version")
        path = shutil.which("codex")
        
        return ToolStatus(
            name="codex",
            installed=True,
            version=version,
            path=path
        )
    
    def detect_spec_kit(self) -> ToolStatus:
        """检测 Spec Kit
        
        Spec Kit 通过 uv tool install 安装，命令名为 specify。
        
        Returns:
            ToolStatus: Spec Kit 的状态信息
        """
        if not self.is_command_available("specify"):
            return ToolStatus(name="spec-kit", installed=False)
        
        version = self.get_command_version("specify", "--version")
        path = shutil.which("specify")
        
        return ToolStatus(
            name="spec-kit",
            installed=True,
            version=version,
            path=path
        )
    
    def detect_bmad(self) -> ToolStatus:
        """检测 BMad Method
        
        BMad Method 通过 npx/bunx 安装，命令名为 bmad。
        
        Returns:
            ToolStatus: BMad Method 的状态信息
        """
        if not self.is_command_available("bmad"):
            return ToolStatus(name="bmad-method", installed=False)
        
        version = self.get_command_version("bmad", "--version")
        path = shutil.which("bmad")
        
        return ToolStatus(
            name="bmad-method",
            installed=True,
            version=version,
            path=path
        )
    
    def detect_all_tools(self) -> Dict[str, ToolStatus]:
        """检测所有工具
        
        Returns:
            Dict[str, ToolStatus]: 工具名称到状态的映射
        """
        tools = {
            "nvm": self.detect_nvm(),
            "node": self.detect_node(),
            "conda": self.detect_conda(),
            "bun": self.detect_bun(),
            "uv": self.detect_uv(),
            "claude-code": self.detect_claude_code(),
            "codex": self.detect_codex(),
            "spec-kit": self.detect_spec_kit(),
            "bmad-method": self.detect_bmad(),
        }
        
        return tools



"""日志记录模块单元测试"""

import pytest
import logging
from pathlib import Path
from unittest.mock import patch, MagicMock

from mono_kickstart.logger import (
    setup_logger,
    get_logger,
    ColoredFormatter,
    SensitiveInfoFilter,
    Colors
)


class TestColoredFormatter:
    """彩色格式器测试"""
    
    def test_format_with_color(self):
        """测试彩色格式化"""
        formatter = ColoredFormatter(fmt='%(levelname)s: %(message)s')
        
        record = logging.LogRecord(
            name='test',
            level=logging.INFO,
            pathname='',
            lineno=0,
            msg='Test message',
            args=(),
            exc_info=None
        )
        
        # Mock isatty 返回 True（支持颜色）
        with patch('sys.stdout.isatty', return_value=True):
            result = formatter.format(record)
            
            # 验证包含颜色代码
            assert Colors.GREEN in result or 'INFO' in result
    
    def test_format_without_color(self):
        """测试无颜色格式化"""
        formatter = ColoredFormatter(fmt='%(levelname)s: %(message)s')
        
        record = logging.LogRecord(
            name='test',
            level=logging.INFO,
            pathname='',
            lineno=0,
            msg='Test message',
            args=(),
            exc_info=None
        )
        
        # Mock isatty 返回 False（不支持颜色）
        with patch('sys.stdout.isatty', return_value=False):
            result = formatter.format(record)
            
            # 验证不包含颜色代码
            assert Colors.GREEN not in result
            assert 'INFO: Test message' in result
    
    def test_different_log_levels(self):
        """测试不同日志级别的颜色"""
        formatter = ColoredFormatter(fmt='%(levelname)s: %(message)s')
        
        levels = [
            (logging.DEBUG, Colors.CYAN),
            (logging.INFO, Colors.GREEN),
            (logging.WARNING, Colors.YELLOW),
            (logging.ERROR, Colors.RED),
            (logging.CRITICAL, Colors.RED),
        ]
        
        with patch('sys.stdout.isatty', return_value=True):
            for level, expected_color in levels:
                record = logging.LogRecord(
                    name='test',
                    level=level,
                    pathname='',
                    lineno=0,
                    msg='Test message',
                    args=(),
                    exc_info=None
                )
                
                result = formatter.format(record)
                # 验证包含对应的颜色代码或级别名称
                assert expected_color in result or logging.getLevelName(level) in result


class TestSensitiveInfoFilter:
    """敏感信息过滤器测试"""
    
    def test_filter_home_directory_in_message(self):
        """测试过滤消息中的主目录路径"""
        filter_obj = SensitiveInfoFilter()
        home_dir = str(Path.home())
        
        record = logging.LogRecord(
            name='test',
            level=logging.INFO,
            pathname='',
            lineno=0,
            msg=f'File saved to {home_dir}/config.yaml',
            args=(),
            exc_info=None
        )
        
        filter_obj.filter(record)
        
        # 验证主目录被替换为 ~
        assert home_dir not in record.msg
        assert '~/config.yaml' in record.msg
    
    def test_filter_home_directory_in_args_tuple(self):
        """测试过滤 args 元组中的主目录路径"""
        filter_obj = SensitiveInfoFilter()
        home_dir = str(Path.home())
        
        record = logging.LogRecord(
            name='test',
            level=logging.INFO,
            pathname='',
            lineno=0,
            msg='File saved to %s',
            args=(f'{home_dir}/config.yaml',),
            exc_info=None
        )
        
        filter_obj.filter(record)
        
        # 验证 args 中的主目录被替换
        assert home_dir not in record.args[0]
        assert '~/config.yaml' in record.args[0]
    
    def test_filter_home_directory_in_args_dict(self):
        """测试过滤 args 字典中的主目录路径"""
        filter_obj = SensitiveInfoFilter()
        home_dir = str(Path.home())
        
        # 创建一个简单的 LogRecord
        record = logging.LogRecord(
            name='test',
            level=logging.INFO,
            pathname='',
            lineno=0,
            msg='Test message',
            args=(),
            exc_info=None
        )
        
        # 手动设置 args 为包含字典的元组
        record.args = ({'path': f'{home_dir}/config.yaml'},)
        
        filter_obj.filter(record)
        
        # 验证 args 字典中的主目录被替换
        assert isinstance(record.args[0], dict)
        assert home_dir not in record.args[0]['path']
        assert '~/config.yaml' in record.args[0]['path']
    
    def test_filter_returns_true(self):
        """测试过滤器总是返回 True"""
        filter_obj = SensitiveInfoFilter()
        
        record = logging.LogRecord(
            name='test',
            level=logging.INFO,
            pathname='',
            lineno=0,
            msg='Test message',
            args=(),
            exc_info=None
        )
        
        result = filter_obj.filter(record)
        
        assert result is True


class TestSetupLogger:
    """日志设置测试"""
    
    def test_setup_logger_creates_logger(self, tmp_path):
        """测试创建日志记录器"""
        logger = setup_logger(
            name='test_logger',
            log_dir=tmp_path
        )
        
        assert logger is not None
        assert logger.name == 'test_logger'
        assert logger.level == logging.DEBUG
    
    def test_setup_logger_creates_handlers(self, tmp_path):
        """测试创建 handlers"""
        logger = setup_logger(
            name='test_logger',
            log_dir=tmp_path
        )
        
        # 验证有两个 handlers（控制台和文件）
        assert len(logger.handlers) == 2
        
        # 验证 handler 类型
        handler_types = [type(h).__name__ for h in logger.handlers]
        assert 'StreamHandler' in handler_types
        assert 'FileHandler' in handler_types
    
    def test_setup_logger_creates_log_file(self, tmp_path):
        """测试创建日志文件"""
        logger = setup_logger(
            name='test_logger',
            log_dir=tmp_path
        )
        
        # 验证日志目录存在
        assert tmp_path.exists()
        
        # 验证日志文件被创建
        log_files = list(tmp_path.glob('mk-*.log'))
        assert len(log_files) == 1
    
    def test_setup_logger_with_custom_levels(self, tmp_path):
        """测试自定义日志级别"""
        logger = setup_logger(
            name='test_logger',
            console_level=logging.WARNING,
            file_level=logging.INFO,
            log_dir=tmp_path
        )
        
        # 验证 handler 级别
        console_handler = logger.handlers[0]
        file_handler = logger.handlers[1]
        
        assert console_handler.level == logging.WARNING
        assert file_handler.level == logging.INFO
    
    def test_setup_logger_adds_filters(self, tmp_path):
        """测试添加过滤器"""
        logger = setup_logger(
            name='test_logger',
            log_dir=tmp_path
        )
        
        # 验证每个 handler 都有过滤器
        for handler in logger.handlers:
            assert len(handler.filters) > 0
            assert any(isinstance(f, SensitiveInfoFilter) for f in handler.filters)
    
    def test_setup_logger_clears_existing_handlers(self, tmp_path):
        """测试清除已有的 handlers"""
        # 第一次设置
        logger1 = setup_logger(name='test_logger', log_dir=tmp_path)
        handler_count1 = len(logger1.handlers)
        
        # 第二次设置（应该清除之前的 handlers）
        logger2 = setup_logger(name='test_logger', log_dir=tmp_path)
        handler_count2 = len(logger2.handlers)
        
        # 验证 handler 数量相同（没有重复添加）
        assert handler_count1 == handler_count2


class TestGetLogger:
    """获取日志记录器测试"""
    
    def test_get_logger_returns_existing_logger(self, tmp_path):
        """测试获取已存在的日志记录器"""
        # 先设置日志记录器
        logger1 = setup_logger(name='test_logger', log_dir=tmp_path)
        
        # 获取日志记录器
        logger2 = get_logger(name='test_logger')
        
        # 验证是同一个日志记录器
        assert logger1 is logger2
    
    def test_get_logger_creates_new_logger_if_not_exists(self):
        """测试获取不存在的日志记录器时自动创建"""
        logger = get_logger(name='new_test_logger')
        
        assert logger is not None
        assert logger.name == 'new_test_logger'
        assert len(logger.handlers) > 0
    
    def test_get_logger_default_name(self):
        """测试默认日志记录器名称"""
        logger = get_logger()
        
        assert logger.name == 'mono_kickstart'


class TestLoggerIntegration:
    """日志记录器集成测试"""
    
    def test_logger_writes_to_console(self, tmp_path, capsys):
        """测试日志写入控制台"""
        logger = setup_logger(
            name='test_logger',
            console_level=logging.INFO,
            log_dir=tmp_path
        )
        
        logger.info("Test info message")
        
        captured = capsys.readouterr()
        assert "Test info message" in captured.out
    
    def test_logger_writes_to_file(self, tmp_path):
        """测试日志写入文件"""
        logger = setup_logger(
            name='test_logger',
            file_level=logging.DEBUG,
            log_dir=tmp_path
        )
        
        logger.debug("Test debug message")
        
        # 查找日志文件
        log_files = list(tmp_path.glob('mk-*.log'))
        assert len(log_files) == 1
        
        # 读取日志文件内容
        log_content = log_files[0].read_text()
        assert "Test debug message" in log_content
    
    def test_logger_filters_sensitive_info(self, tmp_path):
        """测试日志过滤敏感信息"""
        logger = setup_logger(
            name='test_logger',
            log_dir=tmp_path
        )
        
        home_dir = str(Path.home())
        logger.info(f"Config saved to {home_dir}/config.yaml")
        
        # 读取日志文件
        log_files = list(tmp_path.glob('mk-*.log'))
        log_content = log_files[0].read_text()
        
        # 验证主目录被替换为 ~
        assert home_dir not in log_content
        assert "~/config.yaml" in log_content



"""安装编排器的基于属性的测试

本模块使用 Hypothesis 框架测试安装编排器的通用属性。
"""

import tempfile
from pathlib import Path
from unittest.mock import Mock, patch, MagicMock

from hypothesis import given, settings, strategies as st, HealthCheck

from mono_kickstart.config import Config, ToolConfig, RegistryConfig, ProjectConfig
from mono_kickstart.installer_base import InstallReport, InstallResult
from mono_kickstart.orchestrator import InstallOrchestrator, INSTALL_ORDER
from mono_kickstart.platform_detector import PlatformInfo, OS, Arch, Shell


# 定义测试数据生成策略

@st.composite
def platform_info_strategy(draw):
    """生成平台信息策略"""
    os = draw(st.sampled_from([OS.MACOS, OS.LINUX]))
    arch = draw(st.sampled_from([Arch.ARM64, Arch.X86_64]))
    shell = draw(st.sampled_from([Shell.BASH, Shell.ZSH, Shell.FISH]))
    shell_config_file = f"/home/user/.{shell.value}rc"
    
    return PlatformInfo(
        os=os,
        arch=arch,
        shell=shell,
        shell_config_file=shell_config_file
    )


@st.composite
def tool_config_strategy(draw):
    """生成工具配置策略"""
    enabled = draw(st.booleans())
    version = draw(st.one_of(st.none(), st.text(min_size=1, max_size=20)))
    install_via = draw(st.one_of(st.none(), st.sampled_from(['bun', 'npm'])))
    
    return ToolConfig(
        enabled=enabled,
        version=version,
        install_via=install_via
    )


@st.composite
def config_strategy(draw):
    """生成配置策略"""
    # 生成工具配置
    tools = {}
    # 随机选择一些工具启用
    enabled_tools = draw(st.sets(st.sampled_from(INSTALL_ORDER), min_size=0, max_size=len(INSTALL_ORDER)))
    
    for tool_name in INSTALL_ORDER:
        if tool_name in enabled_tools:
            tools[tool_name] = draw(tool_config_strategy())
        else:
            # 未启用的工具
            tools[tool_name] = ToolConfig(enabled=False)
    
    return Config(
        project=ProjectConfig(name=draw(st.one_of(st.none(), st.text(min_size=1, max_size=50)))),
        tools=tools,
        registry=RegistryConfig()
    )


@st.composite
def install_report_strategy(draw, tool_name):
    """生成安装报告策略"""
    result = draw(st.sampled_from([InstallResult.SUCCESS, InstallResult.SKIPPED, InstallResult.FAILED]))
    message = draw(st.text(min_size=1, max_size=100))
    version = draw(st.one_of(st.none(), st.text(min_size=1, max_size=20)))
    error = draw(st.one_of(st.none(), st.text(min_size=1, max_size=100))) if result == InstallResult.FAILED else None
    
    return InstallReport(
        tool_name=tool_name,
        result=result,
        message=message,
        version=version,
        error=error
    )


class TestInstallOrderConsistency:
    """测试属性 1: 工具安装顺序一致性
    
    **Validates: Requirements 1.1**
    """
    
    @given(
        config=config_strategy(),
        platform_info=platform_info_strategy()
    )
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
    def test_install_order_follows_dependencies(self, config, platform_info):
        """验证工具安装顺序始终遵循依赖关系
        
        对于任何工具集合和初始安装状态，安装编排器应该按照定义的依赖顺序
        调用工具安装器。
        
        **Validates: Requirements 1.1**
        """
        orchestrator = InstallOrchestrator(config, platform_info, dry_run=False)
        install_order = orchestrator.get_install_order()
        
        # 验证返回的顺序是 INSTALL_ORDER 的子序列
        # 即保持相对顺序不变
        install_order_indices = []
        for tool in install_order:
            if tool in INSTALL_ORDER:
                install_order_indices.append(INSTALL_ORDER.index(tool))
        
        # 验证索引是递增的（保持原始顺序）
        for i in range(len(install_order_indices) - 1):
            assert install_order_indices[i] < install_order_indices[i + 1], \
                f"工具安装顺序不符合依赖关系: {install_order}"
    
    @given(
        config=config_strategy(),
        platform_info=platform_info_strategy()
    )
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
    def test_only_enabled_tools_in_order(self, config, platform_info):
        """验证只有启用的工具出现在安装顺序中
        
        **Validates: Requirements 1.1**
        """
        orchestrator = InstallOrchestrator(config, platform_info, dry_run=False)
        install_order = orchestrator.get_install_order()
        
        # 验证所有返回的工具都是启用的
        for tool_name in install_order:
            tool_config = config.tools.get(tool_name, ToolConfig())
            assert tool_config.enabled, f"未启用的工具 {tool_name} 出现在安装顺序中"


class TestIdempotency:
    """测试属性 2: 幂等性
    
    **Validates: Requirements 1.2, 6.1, 6.2**
    """
    
    @given(
        config=config_strategy(),
        platform_info=platform_info_strategy()
    )
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture], max_examples=50)
    def test_repeated_execution_same_result(self, config, platform_info):
        """验证重复执行产生相同结果
        
        对于任何初始系统状态，执行初始化命令两次应该产生与执行一次相同的最终状态。
        
        **Validates: Requirements 1.2, 6.1, 6.2**
        """
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            
            # 模拟工具已安装的状态
            with patch.object(InstallOrchestrator, 'install_tool') as mock_install:
                # 第一次执行：模拟所有工具安装成功
                def install_side_effect_1(tool_name):
                    return InstallReport(
                        tool_name=tool_name,
                        result=InstallResult.SUCCESS,
                        message=f"{tool_name} 安装成功",
                        version="1.0.0"
                    )
                
                mock_install.side_effect = install_side_effect_1
                
                orchestrator1 = InstallOrchestrator(config, platform_info, dry_run=False)
                reports1 = orchestrator1.install_all_tools()
                
                # 第二次执行：模拟所有工具已安装（跳过）
                def install_side_effect_2(tool_name):
                    return InstallReport(
                        tool_name=tool_name,
                        result=InstallResult.SKIPPED,
                        message=f"{tool_name} 已安装",
                        version="1.0.0"
                    )
                
                mock_install.side_effect = install_side_effect_2
                
                orchestrator2 = InstallOrchestrator(config, platform_info, dry_run=False)
                reports2 = orchestrator2.install_all_tools()
                
                # 验证两次执行的工具列表相同
                assert set(reports1.keys()) == set(reports2.keys()), \
                    "两次执行的工具列表不同"
                
                # 验证第二次执行时，所有工具都被跳过（因为已安装）
                for tool_name, report in reports2.items():
                    assert report.result == InstallResult.SKIPPED, \
                        f"第二次执行时 {tool_name} 应该被跳过"


class TestFaultTolerance:
    """测试属性 5: 容错性
    
    **Validates: Requirements 1.12, 5.13, 6.3**
    """
    
    @given(
        config=config_strategy(),
        platform_info=platform_info_strategy(),
        # 随机选择一些工具失败
        failed_tools=st.sets(st.sampled_from(INSTALL_ORDER), min_size=0, max_size=3)
    )
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture], max_examples=50)
    def test_continues_after_failure(self, config, platform_info, failed_tools):
        """验证部分工具失败后继续安装其余工具
        
        对于任何工具子集的安装失败，安装编排器应该继续安装其余工具，
        并在最终报告中记录所有失败的工具及其错误信息。
        
        **Validates: Requirements 1.12, 5.13, 6.3**
        """
        with patch.object(InstallOrchestrator, 'install_tool') as mock_install:
            # 模拟部分工具失败
            def install_side_effect(tool_name):
                if tool_name in failed_tools:
                    return InstallReport(
                        tool_name=tool_name,
                        result=InstallResult.FAILED,
                        message=f"{tool_name} 安装失败",
                        error="模拟错误"
                    )
                else:
                    return InstallReport(
                        tool_name=tool_name,
                        result=InstallResult.SUCCESS,
                        message=f"{tool_name} 安装成功",
                        version="1.0.0"
                    )
            
            mock_install.side_effect = install_side_effect
            
            orchestrator = InstallOrchestrator(config, platform_info, dry_run=False)
            reports = orchestrator.install_all_tools()
            
            # 验证所有启用的工具都被尝试安装
            enabled_tools = [t for t in INSTALL_ORDER if config.tools.get(t, ToolConfig()).enabled]
            assert set(reports.keys()) == set(enabled_tools), \
                "并非所有启用的工具都被尝试安装"
            
            # 验证失败的工具有错误信息
            for tool_name in failed_tools:
                if tool_name in reports:
                    report = reports[tool_name]
                    assert report.result == InstallResult.FAILED, \
                        f"{tool_name} 应该标记为失败"
                    assert report.error is not None, \
                        f"{tool_name} 的失败报告应该包含错误信息"


class TestInstallSummaryCompleteness:
    """测试属性 6: 安装摘要完整性
    
    **Validates: Requirements 1.13, 5.14**
    """
    
    @given(
        config=config_strategy(),
        platform_info=platform_info_strategy()
    )
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture], max_examples=50)
    def test_summary_contains_all_tools(self, config, platform_info):
        """验证安装摘要包含所有工具的状态信息
        
        对于任何安装结果集合（成功、跳过、失败），最终摘要应该包含所有工具的状态信息。
        
        **Validates: Requirements 1.13, 5.14**
        """
        with patch.object(InstallOrchestrator, 'install_tool') as mock_install:
            # 模拟随机的安装结果
            def install_side_effect(tool_name):
                import random
                result = random.choice([InstallResult.SUCCESS, InstallResult.SKIPPED, InstallResult.FAILED])
                return InstallReport(
                    tool_name=tool_name,
                    result=result,
                    message=f"{tool_name} {result.value}",
                    version="1.0.0" if result != InstallResult.FAILED else None,
                    error="错误信息" if result == InstallResult.FAILED else None
                )
            
            mock_install.side_effect = install_side_effect
            
            orchestrator = InstallOrchestrator(config, platform_info, dry_run=False)
            reports = orchestrator.install_all_tools()
            
            # 验证所有启用的工具都在报告中
            enabled_tools = [t for t in INSTALL_ORDER if config.tools.get(t, ToolConfig()).enabled]
            assert set(reports.keys()) == set(enabled_tools), \
                "安装摘要缺少某些工具"
            
            # 验证每个报告都包含必要的信息
            for tool_name, report in reports.items():
                assert report.tool_name == tool_name, "工具名称不匹配"
                assert report.result in [InstallResult.SUCCESS, InstallResult.SKIPPED, InstallResult.FAILED], \
                    "安装结果无效"
                assert report.message, "缺少描述信息"
                
                # 如果失败，应该有错误信息
                if report.result == InstallResult.FAILED:
                    assert report.error is not None, f"{tool_name} 失败但没有错误信息"


class TestToolEnabledConfiguration:
    """测试属性 10: 工具启用配置生效
    
    **Validates: Requirements 4.6**
    """
    
    @given(
        config=config_strategy(),
        platform_info=platform_info_strategy()
    )
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
    def test_disabled_tools_not_installed(self, config, platform_info):
        """验证禁用的工具不会被安装
        
        对于任何配置中 tools.<name>.enabled 设置为 false 的工具，
        安装编排器不应该尝试安装该工具。
        
        **Validates: Requirements 4.6**
        """
        orchestrator = InstallOrchestrator(config, platform_info, dry_run=False)
        install_order = orchestrator.get_install_order()
        
        # 验证禁用的工具不在安装顺序中
        for tool_name in INSTALL_ORDER:
            tool_config = config.tools.get(tool_name, ToolConfig())
            if not tool_config.enabled:
                assert tool_name not in install_order, \
                    f"禁用的工具 {tool_name} 不应该出现在安装顺序中"
    
    @given(
        config=config_strategy(),
        platform_info=platform_info_strategy()
    )
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture], max_examples=50)
    def test_only_enabled_tools_attempted(self, config, platform_info):
        """验证只有启用的工具被尝试安装
        
        **Validates: Requirements 4.6**
        """
        with patch.object(InstallOrchestrator, 'install_tool') as mock_install:
            mock_install.return_value = InstallReport(
                tool_name="test",
                result=InstallResult.SUCCESS,
                message="成功"
            )
            
            orchestrator = InstallOrchestrator(config, platform_info, dry_run=False)
            reports = orchestrator.install_all_tools()
            
            # 验证所有被安装的工具都是启用的
            for tool_name in reports.keys():
                tool_config = config.tools.get(tool_name, ToolConfig())
                assert tool_config.enabled, \
                    f"禁用的工具 {tool_name} 不应该被尝试安装"


class TestSingleToolUpgradeIsolation:
    """测试属性 11: 单工具升级隔离性
    
    **Validates: Requirements 5.4**
    """
    
    @given(
        config=config_strategy(),
        platform_info=platform_info_strategy(),
        target_tool=st.sampled_from(INSTALL_ORDER)
    )
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture], max_examples=50)
    def test_single_tool_upgrade_only_affects_target(self, config, platform_info, target_tool):
        """验证单工具升级只影响目标工具
        
        对于任何指定的工具名称，执行 upgrade <tool-name> 应该仅升级该工具，
        不影响其他工具。
        
        **Validates: Requirements 5.4**
        """
        with patch.object(InstallOrchestrator, 'upgrade_tool') as mock_upgrade:
            mock_upgrade.return_value = InstallReport(
                tool_name=target_tool,
                result=InstallResult.SUCCESS,
                message=f"{target_tool} 升级成功",
                version="2.0.0"
            )
            
            orchestrator = InstallOrchestrator(config, platform_info, dry_run=False)
            reports = orchestrator.run_upgrade(tool_name=target_tool)
            
            # 验证只有目标工具被升级
            assert len(reports) == 1, "应该只升级一个工具"
            assert target_tool in reports, f"升级报告中缺少目标工具 {target_tool}"
            
            # 验证 upgrade_tool 只被调用一次，且参数正确
            mock_upgrade.assert_called_once_with(target_tool)


class TestErrorMessageCompleteness:
    """测试属性 12: 错误信息完整性
    
    **Validates: Requirements 6.5**
    """
    
    @given(
        config=config_strategy(),
        platform_info=platform_info_strategy(),
        # 随机选择一些工具失败
        failed_tools=st.sets(st.sampled_from(INSTALL_ORDER), min_size=1, max_size=3)
    )
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture], max_examples=50)
    def test_error_reports_contain_tool_name_and_reason(self, config, platform_info, failed_tools):
        """验证错误报告包含工具名称和错误原因
        
        对于任何安装或升级过程中发生的错误，错误报告应该包含失败的工具名称和错误原因。
        
        **Validates: Requirements 6.5**
        """
        with patch.object(InstallOrchestrator, 'install_tool') as mock_install:
            # 模拟部分工具失败
            def install_side_effect(tool_name):
                if tool_name in failed_tools:
                    return InstallReport(
                        tool_name=tool_name,
                        result=InstallResult.FAILED,
                        message=f"{tool_name} 安装失败",
                        error=f"模拟错误: {tool_name} 无法安装"
                    )
                else:
                    return InstallReport(
                        tool_name=tool_name,
                        result=InstallResult.SUCCESS,
                        message=f"{tool_name} 安装成功"
                    )
            
            mock_install.side_effect = install_side_effect
            
            orchestrator = InstallOrchestrator(config, platform_info, dry_run=False)
            reports = orchestrator.install_all_tools()
            
            # 验证所有失败的工具都有完整的错误信息
            for tool_name in failed_tools:
                if tool_name in reports:
                    report = reports[tool_name]
                    
                    # 验证包含工具名称
                    assert report.tool_name == tool_name, \
                        "错误报告缺少工具名称"
                    
                    # 验证包含错误原因
                    assert report.error is not None, \
                        f"{tool_name} 的错误报告缺少错误原因"
                    assert len(report.error) > 0, \
                        f"{tool_name} 的错误原因为空"
                    
                    # 验证消息不为空
                    assert report.message, \
                        f"{tool_name} 的错误报告缺少描述信息"
    
    @given(
        config=config_strategy(),
        platform_info=platform_info_strategy(),
        failed_tool=st.sampled_from(INSTALL_ORDER)
    )
    @settings(suppress_health_check=[HealthCheck.function_scoped_fixture], max_examples=50)
    def test_upgrade_error_reports_complete(self, config, platform_info, failed_tool):
        """验证升级错误报告的完整性
        
        **Validates: Requirements 6.5**
        """
        with patch.object(InstallOrchestrator, 'upgrade_tool') as mock_upgrade:
            mock_upgrade.return_value = InstallReport(
                tool_name=failed_tool,
                result=InstallResult.FAILED,
                message=f"{failed_tool} 升级失败",
                error=f"升级错误: {failed_tool} 版本不兼容"
            )
            
            orchestrator = InstallOrchestrator(config, platform_info, dry_run=False)
            reports = orchestrator.run_upgrade(tool_name=failed_tool)
            
            # 验证错误报告完整
            assert failed_tool in reports, "升级报告中缺少失败的工具"
            report = reports[failed_tool]
            
            assert report.tool_name == failed_tool, "工具名称不匹配"
            assert report.result == InstallResult.FAILED, "应该标记为失败"
            assert report.error is not None, "缺少错误原因"
            assert len(report.error) > 0, "错误原因为空"
            assert report.message, "缺少描述信息"
